(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit Elemente;

interface
USES Classes, Graphics, SysUtils, Windows,
     ImportConstants, ImportTypes, GeneralConstants, RailyTypes, Math, ULineExporter;

const CLineID  = 100;
      CCSegID  = 200;
      CCnctID  = 300;
      CPolyLineID = 400;
      CGroupID = 1000;

{Define DEBUG}
type EFileCorrupt = class (Exception);

     TBasicAtom = class (TObject)
                    UL, LR    : TSinglePoint;
                    EditState : Word;
                    procedure MoveRel(dx, dy: Double); virtual; abstract;
                    procedure RotateAround(Pivot:TDoublePoint; aw:Double); virtual; abstract;
                    procedure ToStream(var S:TStream); virtual;
                    procedure FromStream(var S:TStream); virtual;
                    procedure Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); virtual; abstract;
                    procedure ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); virtual; abstract;
                    procedure GetBasicRectAround(var aUL, aLR:TDoublePoint); virtual;
                    procedure GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint); virtual; abstract;
                    function Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean; virtual; abstract;
                    function IsCompletelyInRect(A, B: TDoublePoint): boolean;
                    function GetNearestPointAndAngle(Point:TDoublePoint;
                             var Angle:Double; var ResultPoint:TDoublePoint):Double; virtual; abstract;
                    function GetLength:Double; virtual; abstract;
                    destructor Free; virtual;
                  end;

     TLineAtom  = class (TBasicAtom)
                    A, B : TDoublePoint;
                    w    : Double;
                    procedure SetParameter(aA, aB: TDoublePoint; aw:Double);
                    procedure MoveRel(dx, dy: Double); override;
                    procedure RotateAround(Pivot:TDoublePoint; aw:Double); override;
                    procedure ToStream(var S:TStream); override;
                    procedure RectCheckIn;
                    procedure GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint); override;
                    procedure ImportWR1(var S:TStream);
       {$IFDEF XMAS}procedure ExportWR1(var S:TStream); {$ENDIF}
                    procedure FromStream(var S:TStream); override;
                    procedure Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); override;
                    procedure ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);override;
                    function Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean; override;

                    function GetLength:Double; override;
                    function GetlengthIntern:Double;
                    function GetNearestPointAndAngle(Point:TDoublePoint;
                             var Angle:Double; var ResultPoint:TDoublePoint):Double; override;
                  end;

     TCSegAtom  = class (TBasicAtom)
                    Center   : TDoublePoint;
                    r, w, dw : Double;
                    procedure SetParameter(aCenter : TDoublePoint; ar, aw, adw:Double);
                    procedure MoveRel(dx, dy: Double); override;
                    procedure RotateAround(Pivot:TDoublePoint; aw:Double); override;
                    procedure ToStream(var S:TStream); override;
                    procedure RectCheckIn;
                    procedure GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint); override;
                    procedure ImportWR1(var S:TStream);
       {$IFDEF XMAS}procedure ExportWR1(var S:TStream); {$ENDIF}
                    procedure FromStream(var S:TStream); override;
                    procedure Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); override;
                    procedure ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); override;
                    function Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean; override;
                    function GetNearestPointAndAngle(Point:TDoublePoint;
                             var Angle:Double; var ResultPoint:TDoublePoint):Double; override;
                    function GetLength:Double; override;
                 end;
     TCnctAtom = class (TBasicAtom)
                    P : TDoublePoint;
                    w : Double;
                    Nr: longint;
                    procedure MoveRel(dx, dy: Double); override;
                    procedure RotateAround(Pivot:TDoublePoint; aw:Double); override;
                    procedure ToStream(var S:TStream); override;
                    procedure RectCheckIn;
                    procedure GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint); override;
                    procedure ImportWR1(var S:TStream);
       {$IFDEF XMAS}procedure ExportWR1(var S:TStream); {$ENDIF}
                    procedure FromStream(var S:TStream); override;
                    procedure Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); override;
                    procedure ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); override;
                    function Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean; override;
                    function GetNearestPointAndAngle(Point:TDoublePoint;
                             var Angle:Double; var ResultPoint:TDoublePoint):Double; override;
                    function GetLength:Double; override;
                 end;

    TPolyLineAtom  = class (TBasicAtom)
                    Points : TList;
                    FillStyle: TBrushStyle;
                    BrushCol : TColor;
                    PenCol   : TColor;
                    Transparent:boolean;
                    constructor Create;
                    procedure MoveRel(dx, dy: Double); override;
                    procedure RotateAround(Pivot:TDoublePoint; aw:Double); override;
                    procedure ToStream(var S:TStream); override;
                    procedure RectCheckIn;
                    procedure GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint); override;
                    procedure FromStream(var S:TStream); override;

                    procedure ImportWR1(var S:TStream);
       {$IFDEF XMAS}procedure ExportWR1(var S:TStream); {$ENDIF}

                    procedure Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); override;
                    procedure ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);override;
                    function Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean; override;

                    function GetlengthIntern:Double;
                    function GetNearestPointAndAngle(Point:TDoublePoint;
                             var Angle:Double; var ResultPoint:TDoublePoint):Double; override;
                    function GetLength:Double; override;
                    destructor Free;override;
                  end;


     TElement =  class (TObject)
              private
                procedure InitSearch(SearchSize:longint);
              public
                BasicElements : TList;
                ActualSearch  : longint;
                Info          : TElementInfo;
                constructor Create;
                procedure ToStream(var S:TStream);
                procedure GraphicsToStream(var S:TStream);
                procedure FromStream(var S:TStream);
                procedure GraphicsFromStream(var S:TStream);
                procedure MoveRel(dx, dy: Double);
                procedure RotateAround(Pivot:TDoublePoint; aw:Double);
                procedure Draw(X:TCanvas; XS, YS, Winkel, fx, fy:Double; DrawInfo:PDrawInfo);
                procedure ExportIt(X:TLineExport; XS, YS, Winkel, fx, fy: Double; DrawInfo:PDrawInfo);
                procedure SingleDraw(X:TCanvas; XS, YS, Winkel, fx, fy:Double; C:TColor; Nr:longint; DrawInfo:PDrawInfo);
                function ElementHit(ClX, ClY:longint; XS, YS, Winkel, fx, fy:Double; var Nr:longint; DrawInfo:PDrawInfo):boolean;
                function IsHitElementNr(ClX, ClY:longint; XS, YS, Winkel, fx, fy:Double; Nr:longint; DrawInfo:PDrawInfo):boolean;
                procedure GetRectAround0(var TL, DR:TDoublePoint);
                procedure GetRectAround1(XS, YS, Winkel:Double; var TL, DR:TDoublePoint);
                // Import A WINRAILY 1.0x module file
                procedure ImportWR1(var S:TStream);
   {$IFDEF XMAS}procedure ExportWR1(var S:TStream); {$ENDIF}
                procedure UseGravity(var P:TDoublePoint; var A:Double; Radius:Double);
                function GetNearestPointAndAngle(Point:TDoublePoint;
                             var Angle:Double; var ResultPoint:TDoublePoint):Double;
                function GetNrOfConnectors:integer;
                function GetConnectorPos(Nr:integer; var P:TDoublePoint; var Angle:Double):boolean;
                function GetConnectorWorldPos(Nr:integer; XS, YS, Winkel:Double; var P:TDoublePoint; var Angle:Double):boolean;
                function GetNextCnctID:integer;
                function GetLength:Double;
                function HasPolygon:boolean;
                destructor Free;
              end;

     TGroupAtom = class (TBasicAtom)
                    Members : TElement;
//                    procedure SetParameter(aA, aB: TDoublePoint; aw:Double);
                    constructor Create;
                    procedure MoveRel(dx, dy: Double); override;
                    procedure RotateAround(Pivot:TDoublePoint; aw:Double); override;
                    procedure CheckIn;
                    procedure GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint); override;
                    procedure ToStream(var S:TStream); override;
                    procedure FromStream(var S:TStream); override;
                    procedure Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); override;
                    procedure ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); override;
                    function Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean; override;
                    function GetNearestPointAndAngle(Point:TDoublePoint;
                             var Angle:Double; var ResultPoint:TDoublePoint):Double; override;
                    function GetLength:Double; override;
                    destructor Free; override;
                  end;



implementation

var P, PMI:array[0..3] of TPoint;
    PM:TPoint;

function CalcDist(X, Y:TDoublePoint):Double; register;
begin
  try
    Result:=Hypot(ABS(X.X-Y.X), ABS(X.Y-Y.Y));
  except
    Result:=10E300;
  end;
end;

function CalcDistSquare(X, Y:TDoublePoint):Double; register;
begin
  Result:=SQR(X.X-Y.X)+SQR(X.Y-Y.Y);
end;

// rotates A Point around another
function RotatePoint(P, Pivot:TDoublePoint; aw:Double):TDoublePoint;
var Cw, Sw:Extended;
begin
  P.X:=P.X-Pivot.X; P.Y:=P.Y-Pivot.Y;
  SinCos(aw, Sw, Cw);
  Result.X:=(P.X*Cw-P.Y*Sw)+Pivot.X;
  Result.Y:=(P.X*Sw+P.Y*Cw)+Pivot.Y;
end;

//Gemss Robert Sedgewick Seiten 402-407
function iCCW(p0,p1,p2:TPoint):integer; register;
var dx1, dx2, dy1, dy2: longint;
begin
  dx1:=p1.X-p0.X; dy1:=p1.Y-p0.Y;
  dx2:=p2.X-p0.X; dy2:=p2.Y-p0.Y;
  if dx1*dy2>dy1*dx2 then Result:=1
   else if dx1*dy2<dy1*dx2 then begin
     Result:=-1;
   end else begin
     if (dx1*dx2<0) or (dy1*dy2<0) then Result:=-1 else
       if (dx1*dx1 + dy1*dy1) >= (dx2*dx2 + dy2*dy2) then Result:=0 else Result:=-1;
   end;
end;

function iIntersect(A, B, C, d:TPoint):boolean; register;
begin
  Result:=(iCCW(A, B, C)*iCCW(A, B, d)<=0) and
          (iCCW(C, d, A)*iCCW(C, d, B)<=0);
end;

function LineSelected(PL, PA: TPoint): boolean; register;
var i:integer;
begin
  Result:=FALSE;
  for i:=0 TO 3 do if iIntersect(PL, PA, PMI[i], P[i]) then begin
    Result:=True;
    break;
  end;
  // if one Point is within the rectangle and no intersection is detected the other is inside, too
  // and then the line is selected
  if not Result then  Result:=((PL.X>PMI[0].X) and (PL.Y>PMI[0].Y)) and ((PL.X<P[3].X) and (PL.Y<P[3].Y));
end;

procedure FixAngle(var w:Double); register;
begin
  if w<0 then w:=w+(2*Pi*trunc(ABS(w)/(2*Pi)+1));
  if w>0 then w:=w-(2*Pi*trunc(w/(2*Pi)));
end;

//******************************************************************************
// Definition of TBasicAtom
//******************************************************************************
procedure TBasicAtom.ToStream(var S:TStream);
begin
  S.Write(EditState, SizeOf(EditState));
end;

procedure TBasicAtom.GetBasicRectAround(var aUL, aLR: TDoublePoint);
begin
  aUL.X:=UL.X; aUL.Y:=UL.Y;
  aLR.X:=LR.X; aLR.Y:=LR.Y;
end;

function TBasicAtom.IsCompletelyInRect(A, B: TDoublePoint): boolean;
begin
  Result:=((A.X<=UL.X) and (A.Y<=UL.Y) and (B.X>=LR.X) and (B.Y>=LR.Y));
end;

procedure TBasicAtom.FromStream(var S:TStream);
begin
  S.Read(EditState, SizeOf(EditState));
end;

destructor TBasicAtom.Free;
begin
//Nothing
end;


//******************************************************************************
// Definition of TCnctAtom
//******************************************************************************
procedure TCnctAtom.ToStream(var S:TStream);
begin
  INHERITED ToStream(S);
  S.Write(P, SizeOf(P));
  S.Write(w, SizeOf(w));
  S.Write(Nr, SizeOf(Nr));
end;

procedure TCnctAtom.ImportWR1(var S:TStream);
var rP:TRealPoint;
    rw:real;
begin
  S.Read(rP, SizeOf(rP));
  S.Read(rw, SizeOf(rw));
  P.X:=rP.X; P.Y:=rP.Y;
  w:=rw;
  RectCheckIn;
end;

{$IFDEF XMAS}
  procedure TCnctAtom.ExportWR1(var S:TStream);
  var rP:TRealPoint;
      rw:real;
  begin
    rP.X:=P.X; rP.Y:=P.Y;
    rw:=w;
    S.Write(rP, SizeOf(rP));
    S.Write(rw, SizeOf(rw));
  end;
{$ENDIF}


procedure TCnctAtom.RectCheckIn;
const Spurweite=0.1;
begin
  UL.X:=P.X-Spurweite/2;
  LR.X:=P.X+Spurweite/2;
  UL.Y:=P.Y-Spurweite/2;
  LR.Y:=P.Y+Spurweite/2;
end;

procedure TCnctAtom.FromStream(var S:TStream);
begin
  INHERITED FromStream(S);
  S.Read(P, SizeOf(P));
  S.Read(w, SizeOf(w));
  S.Read(Nr, SizeOf(Nr));
  RectCheckIn;
end;

procedure TCnctAtom.MoveRel(dx, dy: Double);
begin
  P.X:=P.X+dx; P.Y:=P.Y+dy;
  RectCheckIn;
end;

procedure TCnctAtom.RotateAround(Pivot:TDoublePoint; aw:Double);
begin
  P:=RotatePoint(P, Pivot, aw);
  w:=w+aw;
  if w<0 then w:=w+(2*Pi*trunc(ABS(w)/(2*Pi)+1));
  if w>0 then w:=w-(2*Pi*trunc(w/(2*Pi)));
  RectCheckIn;
end;

procedure TCnctAtom.Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);
var Mx, My  : Double;
    COrth,
    SOrth   : Extended;
    CAlong,
    SAlong  : Extended;
    SWHalbe : Double;
begin
  SWHalbe:=DrawInfo.ElWidth/2;
  if ((DrawInfo.DoubleDraw and CDrawOuter) >0) and
     not ((DrawInfo.DoubleDraw and CDrawBettung) >0) then SWHalbe:=DrawInfo.Spurweite/2;
  Mx:=(aX+P.X*Cw-P.Y*Sw);
  My:=(aY+P.X*Sw+P.Y*Cw);
  SinCos(aw+w+Pi/2, SOrth, COrth);
  SinCos(aw+w, SAlong, CAlong);

  if (DrawInfo.CnctDraw and CDrawCnctSeparator)>0 then begin
    X.MoveTo(Round((Mx+COrth*SWHalbe)*fx), Round((My+SOrth*SWHalbe)*fy));
    X.LineTo(Round((Mx-COrth*SWHalbe)*fx), Round((My-SOrth*SWHalbe)*fy));
  end;
  if (DrawInfo.CnctDraw and CDrawCnctArrow)>0 then begin
    SWHalbe:=SWHalbe*0.60;
    X.MoveTo(Round((Mx-CAlong*SWHalbe-COrth*SWHalbe)*fx), Round((My-SAlong*SWHalbe-SOrth*SWHalbe)*fy));
    X.LineTo(Round(Mx*fx), Round(My*fy));
    X.LineTo(Round((Mx-CAlong*SWHalbe+COrth*SWHalbe)*fx), Round((My-SAlong*SWHalbe+SOrth*SWHalbe)*fy));
  end;
end;

procedure TCnctAtom.ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);
var Mx, My  : Double;
    COrth,
    SOrth   : Extended;
    CAlong,
    SAlong  : Extended;
    SWHalbe : Double;
begin
  SWHalbe:=DrawInfo.ElWidth/2;
  if ((DrawInfo.DoubleDraw and CDrawOuter) >0) and
     not ((DrawInfo.DoubleDraw and CDrawBettung) >0) then SWHalbe:=DrawInfo.Spurweite/2;
  Mx:=(aX+P.X*Cw-P.Y*Sw);
  My:=(aY+P.X*Sw+P.Y*Cw);
  SinCos(aw+w+Pi/2, SOrth, COrth);
  SinCos(aw+w, SAlong, CAlong);

  if (DrawInfo.CnctDraw and CDrawCnctSeparator)>0 then begin
    X.MoveTo((Mx+COrth*SWHalbe)*fx, (My+SOrth*SWHalbe)*fy);
    X.LineTo((Mx-COrth*SWHalbe)*fx, (My-SOrth*SWHalbe)*fy);
  end;
  if (DrawInfo.CnctDraw and CDrawCnctArrow)>0 then begin
    SWHalbe:=SWHalbe*0.60;
    X.MoveTo((Mx-CAlong*SWHalbe-COrth*SWHalbe)*fx, (My-SAlong*SWHalbe-SOrth*SWHalbe)*fy);
    X.LineTo((Mx*fx), (My*fy));
    X.LineTo((Mx-CAlong*SWHalbe+COrth*SWHalbe)*fx, (My-SAlong*SWHalbe+SOrth*SWHalbe)*fy);
  end;
end;

function TCnctAtom.Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean;
var PL, PA:TPoint;
    Mx, My  : Double;
    COrth,
    SOrth   : Extended;
    CAlong,
    SAlong  : Extended;
    SWHalbe : Double;
begin
  Result:=FALSE;
  SWHalbe:=DrawInfo.ClickBreite;
  Mx:=(aX+P.X*Cw-P.Y*Sw);
  My:=(aY+P.X*Sw+P.Y*Cw);
  SinCos(aw+w+Pi/2, SOrth, COrth);
  SinCos(aw+w, SAlong, CAlong);

  PL.X:=Round((Mx+COrth*SWHalbe)*fx);
  PL.Y:=Round((My+SOrth*SWHalbe)*fy);
  PA.X:=Round((Mx-COrth*SWHalbe)*fx);
  PA.Y:=Round((My-SOrth*SWHalbe)*fy);
  Result:=Result or LineSelected(PL, PA);

  PL.X:=Round((Mx-CAlong*2-COrth*2)*fx);
  PL.Y:=Round((My-SAlong*2-SOrth*2)*fy);
  PA.X:=Round(Mx*fx);
  PA.Y:=Round(My*fy);
  Result:=Result or LineSelected(PL, PA);

  PL.X:=Round((Mx-CAlong*2+COrth*2)*fx);
  PL.Y:=Round((My-SAlong*2+SOrth*2)*fy);
  Result:=Result or LineSelected(PL, PA);
end;

procedure TCnctAtom.GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint);
var TA     : TDoublePoint;
    Sw, Cw : Extended;
const Spurweite=0.1;
begin
  SinCos(Winkel, Sw, Cw);
  TA.X:=(XS+P.X*Cw-P.Y*Sw);
  TA.Y:=(YS+P.X*Sw+P.Y*Cw);
  aUL.X:=TA.X;//-Spurweite/2;
  aUL.Y:=TA.Y;//-Spurweite/2;
  aLR.X:=TA.X;//+Spurweite/2;
  aLR.Y:=TA.Y;//+Spurweite/2;
end;

function TCnctAtom.GetNearestPointAndAngle(Point:TDoublePoint; var Angle:Double; var ResultPoint:TDoublePoint):Double;
begin
  ResultPoint:=P;
  Angle:=w+Pi;
  FixAngle(Angle);
  Result:=CalcDistSquare(Point, P);
end;

function TCnctAtom.GetLength:Double;
begin
  Result:=0;
end;

//******************************************************************************
// Definition of TLineAtom
//******************************************************************************
procedure TLineAtom.SetParameter(aA, aB: TDoublePoint; aw:Double);
begin
  A:=aA; B:=aB; w:=aw;
  RectCheckIn;
end;

function TLineAtom.GetLength:Double;
begin
  Result:=0;
  if EditState and CAtomTrack > 0 then Result:=CalcDist(A, B);
end;

function TLineAtom.GetlengthIntern:Double;
begin
  Result:=CalcDist(A, B);
end;

procedure TLineAtom.MoveRel(dx, dy: Double);
begin
  A.X:=A.X+dx; A.Y:=A.Y+dy;
  B.X:=B.X+dx; B.Y:=B.Y+dy;
  RectCheckIn;
end;

procedure TLineAtom.RotateAround(Pivot:TDoublePoint; aw:Double);
begin
  A:=RotatePoint(A, Pivot, aw);
  B:=RotatePoint(B, Pivot, aw);
  w:=w+aw;
  FixAngle(w);
  RectCheckIn;
end;

procedure TLineAtom.RectCheckIn;
begin
  if A.X<B.X then begin UL.X:=A.X; LR.X:=B.X; end
             else begin UL.X:=B.X; LR.X:=A.X; end;
  if A.Y<B.Y then begin UL.Y:=A.Y; LR.Y:=B.Y; end
             else begin UL.Y:=B.Y; LR.Y:=A.Y; end;
end;

procedure TLineAtom.GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint);
var TA, TB, Null:TDoublePoint;
    Sw, Cw:Extended;
begin
  Null.X:=0; Null.Y:=0;
  SinCos(Winkel, Sw, Cw);
  TA:=RotatePoint(A, Null, Winkel);
  TB:=RotatePoint(B, Null, Winkel);
  if TA.X<TB.X then begin aUL.X:=TA.X+XS; aLR.X:=TB.X+XS; end
             else begin aUL.X:=TB.X+XS; aLR.X:=TA.X+XS; end;
  if TA.Y<TB.Y then begin aUL.Y:=TA.Y+YS; aLR.Y:=TB.Y+YS; end
             else begin aUL.Y:=TB.Y+YS; aLR.Y:=TA.Y+YS; end;
end;


procedure TLineAtom.ToStream(var S:TStream);
begin
  INHERITED ToStream(S);
  S.Write(A, SizeOf(A));
  S.Write(B, SizeOf(B));
  S.Write(w, SizeOf(w));
end;

procedure TLineAtom.FromStream(var S:TStream);
begin
  INHERITED FromStream(S);
  S.Read(A, SizeOf(A));
  S.Read(B, SizeOf(B));
  S.Read(w, SizeOf(w));
  RectCheckIn;
end;

procedure TLineAtom.ImportWR1(var S:TStream);
var rA, rB:TRealPoint;
    rw:real;
begin
  S.Read(rA, SizeOf(rA));
  S.Read(rB, SizeOf(rB));
  S.Read(rw, SizeOf(rw));
  A.X:=rA.X; A.Y:=rA.Y;
  B.X:=rB.X; B.Y:=rB.Y;
  w:=rw;
  RectCheckIn;
end;

{$IFDEF XMAS}
  procedure TLineAtom.ExportWR1(var S:TStream);
  var rA, rB:TRealPoint;
      rw:real;
  begin
    rA.X:=A.X; rA.Y:=A.Y;
    rB.X:=B.X; rB.Y:=B.Y;
    rw:=w;
    S.Write(rA, SizeOf(rA));
    S.Write(rB, SizeOf(rB));
    S.Write(rw, SizeOf(rw));
  end;
{$ENDIF}

procedure TLineAtom.Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);
var COrth, SOrth: Extended;
    Weite:array[0..4] of Double;
    i:integer;
    ANZ:integer;
begin
  ANZ:=-1;
  if ((DrawInfo.DoubleDraw and CDrawInner) >0) or ((EditState and CAtomTrack)=0) then begin
    Inc(ANZ);
    Weite[ANZ]:=0;
  end;

  if ((DrawInfo.DoubleDraw and CDrawOuter) >0) and ((EditState and CAtomTrack)>0) then begin
    Inc(ANZ);
    Weite[ANZ]:=DrawInfo.Spurweite/2;
    Inc(ANZ);
    Weite[ANZ]:=-DrawInfo.Spurweite/2;
  end;

  if ((DrawInfo.DoubleDraw and CDrawBettung) >0) and ((EditState and CAtomTrack)>0) then begin
    Inc(ANZ);
    Weite[ANZ]:=DrawInfo.ElWidth/2;;
    Inc(ANZ);
    Weite[ANZ]:=-DrawInfo.ElWidth/2;;
  end;

  for i:=0 TO ANZ do begin
    SinCos(aw+w+Pi/2, SOrth, COrth);
    X.MoveTo(Round((aX+A.X*Cw-A.Y*Sw+COrth*Weite[i])*fx), Round((aY+A.X*Sw+A.Y*Cw+SOrth*Weite[i])*fy));
    X.LineTo(Round((aX+B.X*Cw-B.Y*Sw+COrth*Weite[i])*fx), Round((aY+B.X*Sw+B.Y*Cw+SOrth*Weite[i])*fy));
  end;
end;

procedure TLineAtom.ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);
var COrth, SOrth: Extended;
    Weite:array[0..4] of Double;
    i:integer;
    ANZ:integer;
begin
  ANZ:=-1;
  if ((DrawInfo.DoubleDraw and CDrawInner) >0) or ((EditState and CAtomTrack)=0) then begin
    Inc(ANZ);
    Weite[ANZ]:=0;
  end;

  if ((DrawInfo.DoubleDraw and CDrawOuter) >0) and ((EditState and CAtomTrack)>0) then begin
    Inc(ANZ);
    Weite[ANZ]:=DrawInfo.Spurweite/2;
    Inc(ANZ);
    Weite[ANZ]:=-DrawInfo.Spurweite/2;
  end;

  if ((DrawInfo.DoubleDraw and CDrawBettung) >0) and ((EditState and CAtomTrack)>0) then begin
    Inc(ANZ);
    Weite[ANZ]:=DrawInfo.ElWidth/2;;
    Inc(ANZ);
    Weite[ANZ]:=-DrawInfo.ElWidth/2;;
  end;

  for i:=0 TO ANZ do begin
    SinCos(aw+w+Pi/2, SOrth, COrth);
    X.MoveTo((aX+A.X*Cw-A.Y*Sw+COrth*Weite[i])*fx, (aY+A.X*Sw+A.Y*Cw+SOrth*Weite[i])*fy);
    X.LineTo((aX+B.X*Cw-B.Y*Sw+COrth*Weite[i])*fx, (aY+B.X*Sw+B.Y*Cw+SOrth*Weite[i])*fy);
  end;
end;

function TLineAtom.Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean;
var PL, PA:TPoint;
begin
  WITH B do begin
    PL.X:=Round((aX+A.X*Cw-A.Y*Sw)*fx); PL.Y:=Round((aY+A.X*Sw+A.Y*Cw)*fy);
    PA.X:=Round((aX+B.X*Cw-B.Y*Sw)*fx); PA.Y:=Round((aY+B.X*Sw+B.Y*Cw)*fy);
    Result:=LineSelected(PL, PA);
//    for i:=0 TO 7 do if iIntersect(PL, PA, PMI[i], P[i]) then Result:=True;
  end;
end;

function TLineAtom.GetNearestPointAndAngle(Point:TDoublePoint; var Angle:Double; var ResultPoint:TDoublePoint):Double;
var ADist,
    BDist     : Double;
begin
  ADist:=CalcDistSquare(Point, A);
  BDist:=CalcDistSquare(Point, B);
  if ADist>BDist then begin ResultPoint:=B; Angle:=w; Result:=BDist; end
                 else begin ResultPoint:=A; Angle:=w+Pi; FixAngle(Angle);
                            Result:=ADist;
                      end;
end;

//******************************************************************************
// Definition of TPolyLineAtom
//******************************************************************************
procedure ClearPointObjectList(X:TList);
var i:integer;
begin
  X.Pack;  // Assure there are no NIL Pointers in the List
  for i:=X.Count-1 downto 0 do begin
    TPointObject(X[i]).Free;
    X.Delete(i);
  end;
  X.Pack;
end;

function MinDouble(X, Y:Double):Double;
begin
  if X<Y then Result:=X else Result:=Y;
end;

function MaxDouble(X, Y:Double):Double;
begin
  if X>Y then Result:=X else Result:=Y;
end;

procedure TPolyLineAtom.ImportWR1(var S:TStream);
begin
end;

{$IFDEF XMAS}
  procedure TPolyLineAtom.ExportWR1(var S:TStream);
  begin
  end;
{$ENDIF}

constructor TPolyLineAtom.Create;
begin
  Points:=TList.Create;
end;

destructor TPolyLineAtom.Free;
begin
  ClearPointObjectList(Points);
  Points.Free;
end;

function TPolyLineAtom.GetlengthIntern:Double;
var i:integer;
begin
  Result:=0;
  for i:=0 TO Points.Count-2 do Result:=Result+CalcDist(TPointObject(Points[i]).P,
                                                        TPointObject(Points[i+1]).P);
end;

function TPolyLineAtom.GetLength:Double;
begin
  Result:=0;
end;

procedure TPolyLineAtom.MoveRel(dx, dy: Double);
var i:integer;
begin
  for i:=0 TO Points.Count-1 do begin
    TPointObject(Points[i]).P.X:=TPointObject(Points[i]).P.X+dx;
    TPointObject(Points[i]).P.Y:=TPointObject(Points[i]).P.Y+dy;
  end;
  RectCheckIn;
end;

procedure TPolyLineAtom.RotateAround(Pivot:TDoublePoint; aw:Double);
var i:integer;
begin
  for i:=0 TO Points.Count-1 do begin
    TPointObject(Points[i]).P:=RotatePoint(TPointObject(Points[i]).P, Pivot, aw)
  end;
  RectCheckIn;
end;

procedure TPolyLineAtom.RectCheckIn;
var i     : integer;
begin
  for i:=0 TO Points.Count-1 do begin
    if i=0 then begin
       UL.X:=TPointObject(Points[i]).P.X;
       UL.Y:=TPointObject(Points[i]).P.Y;
       LR:=UL;
    end else begin
       UL.X:=MinDouble(UL.X, TPointObject(Points[i]).P.X);
       UL.Y:=MinDouble(UL.Y, TPointObject(Points[i]).P.Y);
       LR.X:=MaxDouble(LR.X, TPointObject(Points[i]).P.X);
       LR.Y:=MaxDouble(LR.Y, TPointObject(Points[i]).P.Y);
    end;
  end;
end;

procedure TPolyLineAtom.GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint);
var TA, Null : TDoublePoint;
    Sw, Cw   : Extended;
    i        : integer;
begin
  Null.X:=0; Null.Y:=0;
  SinCos(Winkel, Sw, Cw);
  for i:=0 TO Points.Count-1 do begin
    if i=0 then begin
       TA:=RotatePoint(TPointObject(Points[i]).P, Null, Winkel);
       aUL.X:=TA.X+XS; aUL.Y:=TA.Y+YS;
    end else begin
       TA:=RotatePoint(TPointObject(Points[i]).P, Null, Winkel);
       TA.X:=TA.X+XS; TA.Y:=TA.Y+YS;
       aUL.X:=MinDouble(aUL.X, TA.X);
       aUL.Y:=MinDouble(aUL.Y, TA.Y);
       aLR.X:=MaxDouble(aLR.X, TA.X);
       aLR.Y:=MaxDouble(aLR.Y, TA.Y);
    end;
  end;
end;

procedure TPolyLineAtom.ToStream(var S:TStream);
var C, i:integer;
begin
  INHERITED ToStream(S);
  S.Write(FillStyle, SizeOf(TBrushStyle));
  S.Write(BrushCol, SizeOf(TColor));
  S.Write(PenCol, SizeOf(TColor));
  S.Write(Transparent, SizeOf(boolean));
  Points.Pack;
  C:=Points.Count;
  S.Write(C, SizeOf(C));
  for i:=0 TO C-1 do begin
    S.Write(TPointObject(Points[i]).P, SizeOf(TDoublePoint));
  end;
end;

procedure TPolyLineAtom.FromStream(var S:TStream);
var C, i:integer;
    P:TDoublePoint;
begin
  INHERITED FromStream(S);
  ClearPointObjectList(Points);
  S.Read(FillStyle, SizeOf(TBrushStyle));
  S.Read(BrushCol, SizeOf(TColor));
  S.Read(PenCol, SizeOf(TColor));
  S.Read(Transparent, SizeOf(boolean));
  S.Read(C, SizeOf(C));
  for i:=0 TO C-1 do begin
    S.Read(P, SizeOf(P));
    Points.Add(TPointObject.Create(P.X, P.Y));
  end;
  RectCheckIn;
end;

procedure TPolyLineAtom.Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);
var i:integer;
    A:TDoublePoint;
    PR, L:^TPoint;
    OldS:TBrushStyle;
    OldCB,
    OldCP: TColor;
begin
  if Points.Count<3 then Exit;  // no sense in drawing Nothing
  try
    GetMem(PR, (Points.Count+1)*SizeOf(TPoint));
    L:=PR;
    for i:=0 TO Points.Count-1 do begin
      A:=TPointObject(Points[i]).P;
      L^.X:=Round((aX+A.X*Cw-A.Y*Sw)*fx);
      L^.Y:=Round((aY+A.X*Sw+A.Y*Cw)*fy);
      Inc(L);              
    end;
    A:=TPointObject(Points[0]).P;
    L^.X:=Round((aX+A.X*Cw-A.Y*Sw)*fx);
    L^.Y:=Round((aY+A.X*Sw+A.Y*Cw)*fy);
    OldS:=X.Brush.Style;
    OldCB:=X.Brush.Color;
    OldCP:=X.Pen.Color;
    X.Brush.Style:=FillStyle;
    X.Brush.Color:=BrushCol;
    X.Pen.Color:=PenCol;
    if DrawInfo.selected then X.Pen.Color:=CLRed;
    if Transparent or DrawInfo.ForceTransparent then Windows.Polyline(X.Handle, PR^, Points.Count+1)
                   else Windows.Polygon(X.Handle, PR^, Points.Count);
    X.Brush.Style:=OldS;
    X.Brush.Color:=OldCB;
    X.Pen.Color:=OldCP;
  finally
    FreeMem(PR);
  end;
end;

procedure TPolyLineAtom.ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);
var i:integer;
    A:TDoublePoint;
begin
  if Points.Count<2 then Exit;  // no sense in drawing Nothing
  A:=TPointObject(Points[0]).P;
  X.MoveTo(Round((aX+A.X*Cw-A.Y*Sw)*fx), Round((aY+A.X*Sw+A.Y*Cw)*fy));
  for i:=1 TO Points.Count-1 do begin
    A:=TPointObject(Points[i]).P;
    X.LineTo(Round((aX+A.X*Cw-A.Y*Sw)*fx), Round((aY+A.X*Sw+A.Y*Cw)*fy));
  end;
  A:=TPointObject(Points[0]).P;
  X.LineTo(Round((aX+A.X*Cw-A.Y*Sw)*fx), Round((aY+A.X*Sw+A.Y*Cw)*fy));
end;

function TPolyLineAtom.Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean;
var PA, PB : TPoint;
    B      : TDoublePoint;
    i      : integer;
begin
  Result:=FALSE;
  if Points.Count<2 then Exit;  // no sense in checking Nothing

  B:=TPointObject(Points[0]).P;
  PB.X:=Round((aX+B.X*Cw-B.Y*Sw)*fx); PB.Y:=Round((aY+B.X*Sw+B.Y*Cw)*fy);
  for i:=1 TO Points.Count-1 do begin
    B:=TPointObject(Points[i]).P;
    PA:=PB;
    PB.X:=Round((aX+B.X*Cw-B.Y*Sw)*fx); PB.Y:=Round((aY+B.X*Sw+B.Y*Cw)*fy);
    Result:=Result or LineSelected(PA, PB);
  end;
  PA:=PB;
  B:=TPointObject(Points[0]).P;
  PB.X:=Round((aX+B.X*Cw-B.Y*Sw)*fx); PB.Y:=Round((aY+B.X*Sw+B.Y*Cw)*fy);
  Result:=Result or LineSelected(PA, PB);
end;

function TPolyLineAtom.GetNearestPointAndAngle(Point:TDoublePoint; var Angle:Double; var ResultPoint:TDoublePoint):Double;
{var ADist,
    BDist     : Double;}
begin
     Result:=MaxLongInt;
{//  ADist:=CalcDistSquare(Point, A);
//  BDist:=CalcDistSquare(Point, B);
  if ADist>BDist then begin ResultPoint:=B; Angle:=w; Result:=BDist; end
                 else begin ResultPoint:=A; Angle:=w+Pi; FixAngle(Angle);
                            Result:=ADist;
                      end;
}end;

//******************************************************************************
// Definition of TCSegAtom
//******************************************************************************
procedure TCSegAtom.SetParameter(aCenter : TDoublePoint; ar, aw, adw:Double);
begin
  Center:=aCenter; r:=ar; w:=aw; dw:=adw;
  RectCheckIn;
end;

procedure TCSegAtom.MoveRel(dx, dy: Double);
begin
  Center.X:=Center.X+dx; Center.Y:=Center.Y+dy;
  RectCheckIn;
end;

function TCSegAtom.GetLength:Double;
begin
  FixAngle(dw);
  Result:=0;
  if EditState and CAtomTrack > 0 then Result:=r*dw
end;

procedure TCSegAtom.RotateAround(Pivot:TDoublePoint; aw:Double);
begin
  Center:=RotatePoint(Center, Pivot, aw);
  w:=w-aw;
  FixAngle(w);
  RectCheckIn;
end;

procedure TCSegAtom.RectCheckIn;
var A, B: TDoublePoint;
    wm, wdw:Double;
begin
  FixAngle(w);
  FixAngle(dw);

  A.X:=Center.X+Cos(w)*r;
  A.Y:=Center.Y-Sin(w)*r;
  B.X:=Center.X+Cos(w+dw)*r;
  B.Y:=Center.Y-Sin(w+dw)*r;
  if A.X<B.X then begin UL.X:=A.X; LR.X:=B.X end else begin UL.X:=B.X; LR.X:=A.X end;
  if A.Y<B.Y then begin UL.Y:=A.Y; LR.Y:=B.Y end else begin UL.Y:=B.Y; LR.Y:=A.Y end;

  // could be optimized if not testing (w<=Pi/2) and (w>=Pi/2)
  // but this way it is better TO Read
  wdw:=w+dw;
  wm:=wdw-2*Pi;
  // Top special case
  if ((w<=Pi/2) and (wdw>=Pi/2)) or ((w>=Pi/2) and (wm>=Pi/2)) then UL.Y:=Center.Y-r;

  // Left special case
  if ((w<=Pi) and (wdw>=Pi)) or ((w>=Pi) and (wm>=Pi)) then UL.X:=Center.X-r;

  // Bottom special case
  if ((w<=3*Pi/2) and (wdw>=3*Pi/2)) or ((w>=3*Pi/2) and (wm>=3*Pi/2)) then LR.Y:=Center.Y+r;

  // Right special case
  if (wdw>2*Pi) then LR.X:=Center.X+r;
end;

procedure TCSegAtom.GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint);
var A, B, C    : TDoublePoint;
    wm, wdw    : Double;
//    Sw, Cw     : Extended;
    tw:Double;
    Null:TDoublePoint;
begin
  Null.X:=0; Null.Y:=0;
  tw:=w-Winkel;
  FixAngle(tw);
  FixAngle(dw);
  C:=RotatePoint(Center, Null, Winkel);

  A.X:=C.X+Cos(tw)*r;
  A.Y:=C.Y-Sin(tw)*r;
  B.X:=C.X+Cos(tw+dw)*r;
  B.Y:=C.Y-Sin(tw+dw)*r;
  if A.X<B.X then begin aUL.X:=A.X; aLR.X:=B.X end else begin aUL.X:=B.X; aLR.X:=A.X end;
  if A.Y<B.Y then begin aUL.Y:=A.Y; aLR.Y:=B.Y end else begin aUL.Y:=B.Y; aLR.Y:=A.Y end;


  // could be optimized if not testing (tw<=Pi/2) and (tw>=Pi/2)
  // but this way it is better TO Read
  wdw:=tw+dw;
  wm:=wdw-2*Pi;
  // Top special case
  if ((tw<=Pi/2) and (wdw>=Pi/2)) or ((tw>=Pi/2) and (wm>=Pi/2)) then aUL.Y:=C.Y-r;

  // Left special case
  if ((tw<=Pi) and (wdw>=Pi)) or ((tw>=Pi) and (wm>=Pi)) then aUL.X:=C.X-r;

  // Bottom special case
  if ((tw<=3*Pi/2) and (wdw>=3*Pi/2)) or ((tw>=3*Pi/2) and (wm>=3*Pi/2)) then aLR.Y:=C.Y+r;

  // Right special case
  if (wdw>2*Pi) then aLR.X:=C.X+r;
  aUL.X:=aUL.X+XS;
  aUL.Y:=aUL.Y+YS;
  aLR.X:=aLR.X+XS;
  aLR.Y:=aLR.Y+YS;
end;


procedure TCSegAtom.ToStream(var S:TStream);
begin
  INHERITED ToStream(S);
  S.Write(Center, SizeOf(Center));
  S.Write(r, SizeOf(r));
  S.Write(w, SizeOf(w));
  S.Write(dw, SizeOf(dw));
end;

procedure TCSegAtom.FromStream(var S:TStream);
begin
  INHERITED FromStream(S);
  S.Read(Center, SizeOf(Center));
  S.Read(r, SizeOf(r));
  S.Read(w, SizeOf(w));
  S.Read(dw, SizeOf(dw));
  RectCheckIn;
end;

procedure TCSegAtom.ImportWR1(var S:TStream);
var rr,rw,rdw:real;
    rCenter:TRealPoint;
begin
  S.Read(rCenter, SizeOf(rCenter));
  S.Read(rr, SizeOf(rr));
  S.Read(rw, SizeOf(rw));
  S.Read(rdw, SizeOf(rdw));
  Center.X:=rCenter.X; Center.Y:=rCenter.Y;
  r:=rr; w:=rw; dw:=rdw;
  RectCheckIn;
end;

{$IFDEF XMAS}
  procedure TCSegAtom.ExportWR1(var S:TStream);
  var rr,rw,rdw:real;
      rCenter:TRealPoint;
  begin
    rCenter.X:=Center.X; rCenter.Y:=Center.Y;
    rr:=r; rw:=w; rdw:=dw;
    S.Write(rCenter, SizeOf(rCenter));
    S.Write(rr, SizeOf(rr));
    S.Write(rw, SizeOf(rw));
    S.Write(rdw, SizeOf(rdw));
  end;
{$ENDIF}

procedure TCSegAtom.Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);
var Tx, Ty, TAx, TAy, Nx, Ny{, RPlusSHalf, RMinusSHalf}: Double;
    TiX1, TiY1, TiX2, TiY2:integer;
    Omega:Double;
    NoAnX, NoAnY:Double;
    ANZ:integer;
    Weite:array[0..4] of Double;
    i:integer;
begin
  ANZ:=-1;
  if ((DrawInfo.DoubleDraw and CDrawInner) >0) or ((EditState and CAtomTrack)=0) then begin
    Inc(ANZ);
    Weite[ANZ]:=r;
  end;

  if ((DrawInfo.DoubleDraw and CDrawOuter) >0) and ((EditState and CAtomTrack)>0) then begin
    Inc(ANZ);
    Weite[ANZ]:=r+DrawInfo.Spurweite/2;
    Inc(ANZ);
    Weite[ANZ]:=r-DrawInfo.Spurweite/2;
  end;

  if ((DrawInfo.DoubleDraw and CDrawBettung) >0) and ((EditState and CAtomTrack)>0) then begin
    Inc(ANZ);
    Weite[ANZ]:=r+DrawInfo.ElWidth/2;;
    Inc(ANZ);
    Weite[ANZ]:=r-DrawInfo.ElWidth/2;;
  end;

  for i:=0 TO ANZ do begin
    {Cw und Sw sind Cosinus und Sinus des Winkels w der Gesamtschiene.
     Cw un Sw werden im bergeordneten TAnObject Objekt jeweils
     vor dem Anzeigen gerechnet und an die Kurven und Geraden bergeben}
    Tx:=(aX+Center.X*Cw-Center.Y*Sw)*fx;
    Ty:=(aY+Center.X*Sw+Center.Y*Cw)*fy;
    TAx:=Center.X+Cos(w)*(Weite[i]);
    TAy:=Center.Y-Sin(w)*(Weite[i]);
    Nx:=Center.X+Cos(w+dw)*(Weite[i]);
    Ny:=Center.Y-Sin(w+dw)*(Weite[i]);

    TiX1:=Round((aX+TAx*Cw-TAy*Sw)*fx);
    TiY1:=Round((aY+TAx*Sw+TAy*Cw)*fy);
    TiX2:=Round((aX+Nx*Cw-Ny*Sw)*fx);
    TiY2:=Round((aY+Nx*Sw+Ny*Cw)*fy);
    if ((ABS(TiX1-TiX2)<=2) and (ABS(TiY1-TiY2)<=2)) or DrawInfo.Exact then begin
      X.MoveTo(Round((aX+TAx*Cw-TAy*Sw)*fx), Round((aY+TAx*Sw+TAy*Cw)*fy));
      Omega:=w;
      while Omega<w+dw do begin
        NoAnX:=Center.X+Cos(Omega)*(Weite[i]);
        NoAnY:=Center.Y-Sin(Omega)*(Weite[i]);
        Omega:=Omega+Pi/DrawInfo.Precision;
        X.LineTo(Round((aX+NoAnX*Cw-NoAnY*Sw)*fx), Round((aY+NoAnX*Sw+NoAnY*Cw)*fy));
      end;
      X.LineTo(Round((aX+Nx*Cw-Ny*Sw)*fx), Round((aY+Nx*Sw+Ny*Cw)*fy));
    end else begin
      X.Arc(Round(Tx-(Weite[i])*fx), Round(Ty-(Weite[i])*fy),
            Round(Tx+(Weite[i])*fx), Round(Ty+(Weite[i])*fy),
            TiX1, TiY1, TiX2, TiY2);
    end;
  end;

  {$IFDEF DEBUG}
  X.Brush.Style:=bsClear;
  X.rectangle(Round((aX+UL.X)*fx), Round((aY+UL.Y)*fy), Round((aX+LR.X)*fx), Round((aY+LR.Y)*fy));
  {$ENDIF}
end;

procedure TCSegAtom.ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); 
var TAx, TAy, Nx, Ny{, RPlusSHalf, RMinusSHalf}: Double;
    Omega:Double;
    NoAnX, NoAnY:Double;
    ANZ:integer;
    Weite:array[0..4] of Double;
    i:integer;
begin
  ANZ:=-1;
  if ((DrawInfo.DoubleDraw and CDrawInner) >0) or ((EditState and CAtomTrack)=0) then begin
    Inc(ANZ);
    Weite[ANZ]:=r;
  end;

  if ((DrawInfo.DoubleDraw and CDrawOuter) >0) and ((EditState and CAtomTrack)>0) then begin
    Inc(ANZ);
    Weite[ANZ]:=r+DrawInfo.Spurweite/2;
    Inc(ANZ);
    Weite[ANZ]:=r-DrawInfo.Spurweite/2;
  end;

  if ((DrawInfo.DoubleDraw and CDrawBettung) >0) and ((EditState and CAtomTrack)>0) then begin
    Inc(ANZ);
    Weite[ANZ]:=r+DrawInfo.ElWidth/2;;
    Inc(ANZ);
    Weite[ANZ]:=r-DrawInfo.ElWidth/2;;
  end;

  for i:=0 TO ANZ do begin
    {Cw und Sw sind Cosinus und Sinus des Winkels w der Gesamtschiene.
     Cw un Sw werden im bergeordneten TAnObject Objekt jeweils
     vor dem Anzeigen gerechnet und an die Kurven und Geraden bergeben}
//    Tx:=(aX+Center.X*Cw-Center.Y*Sw)*fx;
//    Ty:=(aY+Center.X*Sw+Center.Y*Cw)*fy;
    TAx:=Center.X+Cos(w)*(Weite[i]);
    TAy:=Center.Y-Sin(w)*(Weite[i]);
    Nx:=Center.X+Cos(w+dw)*(Weite[i]);
    Ny:=Center.Y-Sin(w+dw)*(Weite[i]);

//    TiX1:=Round((aX+TAx*Cw-TAy*Sw)*fx);
//    TiY1:=Round((aY+TAx*Sw+TAy*Cw)*fy);
//    TiX2:=Round((aX+Nx*Cw-Ny*Sw)*fx);
//    TiY2:=Round((aY+Nx*Sw+Ny*Cw)*fy);
    X.MoveTo(((aX+TAx*Cw-TAy*Sw)*fx), ((aY+TAx*Sw+TAy*Cw)*fy));
    Omega:=w;
    while Omega<w+dw do begin
      NoAnX:=Center.X+Cos(Omega)*(Weite[i]);
      NoAnY:=Center.Y-Sin(Omega)*(Weite[i]);
      Omega:=Omega+Pi/DrawInfo.Precision;
      X.LineTo(((aX+NoAnX*Cw-NoAnY*Sw)*fx), ((aY+NoAnX*Sw+NoAnY*Cw)*fy));
    end;
    X.LineTo(((aX+Nx*Cw-Ny*Sw)*fx), ((aY+Nx*Sw+Ny*Cw)*fy));
  end;
end;

function TCSegAtom.Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean;
var TAx, TAy, Nx, Ny: Double;
    Omega:Double;
    NoAnX, NoAnY:Double;
    f:Double;
    PL, PA:TPoint;
begin
  Result:=FALSE;
//  if ABS(Hypot((aX+Center.X)*fx-ClX, (aY+Center.Y)*fy-ClY)-r) > DrawInfo.Spurweite/2 then Exit;

  TAx:=Center.X+Cos(w)*r;
  TAy:=Center.Y-Sin(w)*r;
  Nx:=Center.X+Cos(w+dw)*r;
  Ny:=Center.Y-Sin(w+dw)*r;

  PL.X:=Round((aX+TAx*Cw-TAy*Sw)*fx);
  PL.Y:=Round((aY+TAx*Sw+TAy*Cw)*fy);

  Omega:=w;
  f:=SQRT(ABS(r*dw*fx));
//      WriteLn(f:2:2);
  while Omega<w+dw do begin
    NoAnX:=Center.X+Cos(Omega)*r;
    NoAnY:=Center.Y-Sin(Omega)*r;
    Omega:=Omega+Pi/f;
    PA.X:=Round((aX+NoAnX*Cw-NoAnY*Sw)*fx);
    PA.Y:=Round((aY+NoAnX*Sw+NoAnY*Cw)*fy);
    if LineSelected(PL, PA) then Result:=True;
//    for i:=0 TO 8 do if iIntersect(PL, PA, PMI[i], P[i]) then Result:=True;
    PL:=PA;
  end;
   PA.X:=Round((aX+Nx*Cw-Ny*Sw)*fx); PA.Y:=Round((aY+Nx*Sw+Ny*Cw)*fy);
  if LineSelected(PL, PA) then Result:=True;
//   for i:=0 TO 8 do if iIntersect(PL, PA, PMI[i], P[i]) then Result:=True;
end;

function TCSegAtom.GetNearestPointAndAngle(Point:TDoublePoint; var Angle:Double; var ResultPoint:TDoublePoint):Double;
var ADist,
    BDist     : Double;
    APoint,
    BPoint    : TDoublePoint;
    TempPoint : TDoublePoint;
    TempDist  : Double;
    TempAngle : Double;
begin
// Probably wrong
  APoint.X:=Center.X+Cos(w)*r;APoint.Y:=Center.Y-Sin(w)*r;
  BPoint.X:=Center.X+Cos(w+dw)*r;BPoint.Y:=Center.Y-Sin(w+dw)*r;

  ADist:=CalcDistSquare(Point, APoint);
  BDist:=CalcDistSquare(Point, BPoint);
  if ADist>BDist then begin TempPoint:=BPoint; TempAngle:=-w-dw-Pi/2; TempDist:=BDist; end
                 else begin TempPoint:=APoint; TempAngle:=-w+Pi/2;
                            TempDist:=ADist;
                      end;
// Reuse A
  ADist:=CalcDistSquare(Point, Center);
  if ADist>TempDist then begin
                           ResultPoint:=TempPoint;
                           Angle:=TempAngle;
                           FixAngle(Angle);
                           Result:=TempDist;
                    end else begin
                           ResultPoint:=Center;
                           Angle:=0;
                           Result:=ADist;
                    end;
end;


//******************************************************************************
// Definition of TGroupAtom
//******************************************************************************
constructor TGroupAtom.Create;
begin
  Members:=TElement.Create;
end;

procedure TGroupAtom.ToStream(var S:TStream);
begin
  Members.ToStream(S);
end;

procedure TGroupAtom.FromStream(var S:TStream);
begin
  Members.FromStream(S);
  CheckIn;
end;

procedure TGroupAtom.Draw(X:TCanvas; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo);
begin
  Members.Draw(X, aX, aY, aw, fx, fy, DrawInfo);
end;

procedure TGroupAtom.ExportIt(X:TLineExport; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo); 
begin
  Members.ExportIt(X, aX, aY, aw, fx, fy, DrawInfo);
end;


function TGroupAtom.Hit(ClX, ClY:longint; aX, aY, Sw, Cw, aw, fx, fy: Double; DrawInfo:PDrawInfo):boolean;
var Dummy:longint;
begin
  Result:=Members.ElementHit(ClX, ClY, aX, aY, aw, fx, fy, Dummy, DrawInfo);
end;

function TGroupAtom.GetLength:Double;
begin
  Result:=Members.GetLength;
end;

procedure TGroupAtom.MoveRel(dx, dy: Double);
begin
  Members.MoveRel(dx, dy);
  CheckIn;
end;

procedure TGroupAtom.RotateAround(Pivot:TDoublePoint; aw:Double);
begin
  Members.RotateAround(Pivot, aw);
  CheckIn;
end;


procedure TGroupAtom.CheckIn;
var dUL, dLR:TDoublePoint;
begin
  Members.GetRectAround0(dUL, dLR);
  UL.X:=dUL.X; UL.Y:=dUL.Y;
  LR.X:=dLR.X; LR.Y:=dLR.Y;
end;

procedure TGroupAtom.GetExtendedRectAround(XS, YS, Winkel:Double; var aUL, aLR:TDoublePoint);
var dUL, dLR:TDoublePoint;
begin
  Members.GetRectAround1(XS, YS, Winkel, dUL, dLR);
  aUL.X:=dUL.X; aUL.Y:=dUL.Y;
  aLR.X:=dLR.X; aLR.Y:=dLR.Y;
end;

function TGroupAtom.GetNearestPointAndAngle(Point:TDoublePoint; var Angle:Double; var ResultPoint:TDoublePoint):Double;
begin
  Result:=Members.GetNearestPointAndAngle(Point, Angle, ResultPoint);
end;

destructor TGroupAtom.Free;
begin
  Members.Free;
end;

//******************************************************************************
// Definition of TElement
//******************************************************************************
constructor TElement.Create;
begin
  BasicElements:=TList.Create;
  ActualSearch:=0;
end;


// TO save memory in ElementEditor allow saving the Graphics only (undo)
procedure TElement.GraphicsToStream(var S:TStream);
var C, i : longint;
    B    : TBasicAtom;
    ID   : longint;
begin
  BasicElements.Pack;
  C:=BasicElements.Count;
  S.Write(C, SizeOf(C));
  for i:=0 TO C-1 do begin
    B:=BasicElements.Items[i];

    // if the actual element is A line
    if B is TLineAtom then begin
      ID:=CLineID;
      S.Write(ID, SizeOf(ID));
      TLineAtom(B).ToStream(S);
      Continue;
    end;

    // if the actual element is A Polygone
    if B is TPolyLineAtom then begin
      ID:=CPolyLineID;
      S.Write(ID, SizeOf(ID));
      TPolyLineAtom(B).ToStream(S);
      Continue;
    end;

    // if the actual element is A Connector
    if B is TCnctAtom then begin
      ID:=CCnctID;
      S.Write(ID, SizeOf(ID));
      TCnctAtom(B).ToStream(S);
      Continue;
    end;

    // if the actual element is A Group
    if B is TGroupAtom then begin
      ID:=CGroupID;
      S.Write(ID, SizeOf(ID));
      TGroupAtom(B).ToStream(S);
      Continue;
    end;

    // if the acutal element is A circle-segment
    if B is TCSegAtom then begin
      ID:=CCSegID;
      S.Write(ID, SizeOf(ID));
      TCSegAtom(B).ToStream(S);
      Continue;
    end;
  end;
end;

function TElement.HasPolygon:boolean;
var i:integer;
begin
  Result:=FALSE;
  for i:=0 TO BasicElements.Count-1 do begin
    Result:=Result or (TBasicAtom(BasicElements.Items[i]) is TPolyLineAtom);
  end;
end;

procedure TElement.ToStream(var S:TStream);
begin
  S.Write(Info, SizeOf(Info));
  GraphicsToStream(S);
end;

procedure TElement.GraphicsFromStream(var S:TStream);
var C, i, ID : longint;
    B : TBasicAtom;
begin
  WITH BasicElements do begin
    Pack;
    while Count>0 do Remove(First);
  end;
  S.Read(C, SizeOf(C));
  for i:=0 TO C-1 do begin
    S.Read(ID, SizeOf(longint));
    case ID of
      CLineID : B:=TLineAtom.Create;
      CCSegID : B:=TCSegAtom.Create;
      CCnctID : B:=TCnctAtom.Create;
      CGroupID: B:=TGroupAtom.Create;
      CPolyLineID : B:=TPolyLineAtom.Create;
      else begin
        raise EFileCorrupt.Create('Illegal atom in procedure TElement.FromStream(var S:TStream)');
      end;
    end;
    B.FromStream(S);
    BasicElements.Add(B);
  end;
end;

procedure TElement.FromStream(var S:TStream);
begin
  S.Read(Info, SizeOf(Info));
  GraphicsFromStream(S);
end;

procedure TElement.MoveRel(dx, dy: Double);
var i:longint;
begin
  for i:=0 TO BasicElements.Count-1 do begin
    TBasicAtom(BasicElements.Items[i]).MoveRel(dx, dy);
  end;
end;

procedure TElement.RotateAround(Pivot:TDoublePoint; aw:Double);
var i:longint;
begin
  for i:=0 TO BasicElements.Count-1 do begin
    TBasicAtom(BasicElements.Items[i]).RotateAround(Pivot, aw);
  end;
end;

procedure TElement.Draw(X:TCanvas; XS, YS, Winkel, fx, fy:Double; DrawInfo:PDrawInfo);
var i{, Prev}:longint;
    Cw, Sw:Extended;
begin
  SinCos(Winkel, Sw, Cw);
{  X.Pen.Style:=psDot;
  Prev:=SetBKMode(X.Handle, Transparent); }
  for i:=0 TO BasicElements.Count-1 do begin
    TBasicAtom(BasicElements.Items[i]).Draw(X, XS, YS, Sw, Cw, Winkel, fx, fy, DrawInfo);
  end;
{  SetBKMode(X.Handle, Prev);
  X.Pen.Style:=psSolid;}
end;

procedure TElement.ExportIt(X:TLineExport; XS, YS, Winkel, fx, fy: Double; DrawInfo:PDrawInfo);
var i:longint;
    Cw, Sw:Extended;
begin
  SinCos(Winkel, Sw, Cw);
  for i:=0 TO BasicElements.Count-1 do begin
    TBasicAtom(BasicElements.Items[i]).ExportIt(X, XS, YS, Sw, Cw, Winkel, fx, fy, DrawInfo);
  end;
end;


procedure TElement.SingleDraw(X:TCanvas; XS, YS, Winkel, fx, fy:Double; C:TColor; Nr:longint; DrawInfo:PDrawInfo);
var B:TBasicAtom;
    Cw, Sw:Extended;
    TempColor:TColor;
begin
  SinCos(Winkel, Sw, Cw);
  B:=BasicElements.Items[Nr];
  TempColor:=X.Pen.Color;
  X.Pen.Color:=C;
  B.Draw(X, XS, YS, Sw, Cw, Winkel, fx, fy, DrawInfo);
  X.Pen.Color:=TempColor;
end;

procedure TElement.InitSearch(SearchSize:longint);
begin
  PMI[0].Y:=PM.Y-SearchSize; P[0].Y:=PM.Y+SearchSize;
  PMI[0].X:=PM.X-SearchSize; P[0].X:=PM.X-SearchSize;
  PMI[1].Y:=PM.Y-SearchSize; P[1].Y:=PM.Y+SearchSize;
  PMI[1].X:=PM.X+SearchSize; P[1].X:=PM.X+SearchSize;
  PMI[2].Y:=PM.Y-SearchSize; P[2].Y:=PM.Y-SearchSize;
  PMI[2].X:=PM.X-SearchSize; P[2].X:=PM.X+SearchSize;
  PMI[3].Y:=PM.Y+SearchSize; P[3].Y:=PM.Y+SearchSize;
  PMI[3].X:=PM.X-SearchSize; P[3].X:=PM.X+SearchSize;
end;

function TElement.ElementHit(ClX, ClY:longint; XS, YS, Winkel, fx, fy:Double; var Nr:longint; DrawInfo:PDrawInfo):boolean;
var i:longint;
    B:TBasicAtom;
    Cw, Sw:Extended;
begin
  Result:=FALSE;
  SinCos(Winkel, Sw, Cw);
  PM.X:=ClX; PM.Y:=ClY;
  if DrawInfo.ClickBreite=0 then DrawInfo.ClickBreite:=5;
  InitSearch(Round(fx*DrawInfo.ClickBreite));

  for i:=0 TO BasicElements.Count-1 do begin
    ActualSearch:=(ActualSearch+1) MOD BasicElements.Count;
    B:=BasicElements.Items[ActualSearch];
    if B.Hit(ClX, ClY, XS, YS, Sw, Cw, Winkel, fx, fy, DrawInfo) then begin
        Result:=True;
        Nr:=ActualSearch;
        break;
    end;
  end;
end;

function TElement.GetLength:Double;
var i:integer;
    B:TBasicAtom;
begin
  Result:=0;
  for i:=0 TO BasicElements.Count-1 do begin
    B:=BasicElements.Items[i];
    Result:=Result+B.GetLength;
  end;
end;

function TElement.IsHitElementNr(ClX, ClY:longint; XS, YS, Winkel, fx, fy:Double; Nr:longint; DrawInfo:PDrawInfo):boolean;
var B:TBasicAtom;
    Cw, Sw:Extended;
begin
  SinCos(Winkel, Sw, Cw);
  PM.X:=ClX; PM.Y:=ClY;
  InitSearch(Round(fx*DrawInfo.ClickBreite));
  Result:=FALSE;
  if (Nr>=0) and (Nr<BasicElements.Count) then begin
    B:=BasicElements.Items[Nr];
    Result:=B.Hit(ClX, ClY, XS, YS, Sw, Cw, Winkel, fx, fy, DrawInfo);
  end;
end;


procedure TElement.GetRectAround0(var TL, DR:TDoublePoint);
var i:longint;
    B:TBasicAtom;
    First:boolean;
    P, Q: TDoublePoint;
begin
  First:=True;
  TL.X:=0; TL.Y:=0; DR:=TL;
  for i:=0 TO BasicElements.Count-1 do begin
    B:=BasicElements.Items[i];
    B.GetBasicRectAround(P, Q);
    if First then begin TL:=P; DR:=Q; First:=FALSE; end;
    if TL.X>P.X then TL.X:=P.X;
    if TL.Y>P.Y then TL.Y:=P.Y;
    if DR.X<Q.X then DR.X:=Q.X;
    if DR.Y<Q.Y then DR.Y:=Q.Y;
  end;
end;

procedure TElement.GetRectAround1(XS, YS, Winkel:Double; var TL, DR:TDoublePoint);
var i:longint;
    B:TBasicAtom;
    First:boolean;
    P, Q: TDoublePoint;
begin
  First:=True;
  TL.X:=0; TL.Y:=0; DR:=TL;
  for i:=0 TO BasicElements.Count-1 do begin
    B:=BasicElements.Items[i];
// Removed 27.06.1997 could cause floatingpoint errors when 1. atom is TCnctAtom
//    if {(not (B is TCSegAtom)) and} (not (B is TCnctAtom)) {and (not (B is TLineAtom)) } then
      B.GetExtendedRectAround(XS, YS, Winkel, P, Q);
    if First then begin
      if P.X<Q.X then begin TL.X:=P.X; DR.X:=Q.X; end else begin TL.X:=Q.X; DR.X:=P.X; end;
      if P.Y<Q.Y then begin TL.Y:=P.Y; DR.Y:=Q.Y; end else begin TL.Y:=Q.Y; DR.Y:=P.Y; end;
      First:=FALSE;
    end;
    if TL.X>P.X then TL.X:=P.X;
    if TL.Y>P.Y then TL.Y:=P.Y;
    if DR.X<Q.X then DR.X:=Q.X;
    if DR.Y<Q.Y then DR.Y:=Q.Y;
    if TL.X>Q.X then TL.X:=Q.X;
    if TL.Y>Q.Y then TL.Y:=Q.Y;
    if DR.X<P.X then DR.X:=P.X;
    if DR.Y<P.Y then DR.Y:=P.Y;
  end;
end;

procedure TElement.ImportWR1(var S:TStream);
var Number    : integer;
    i         : integer;
    B         : TBasicAtom;
    BNumber   : Byte;
    ElInfo    : TImportElementInfo;
    Uebergang : array[0..25] of CHAR;

// Eat eats A ObjType from A TStreamRec (BP7.0)
  procedure Eat(w:Word);
  var Dummy:Word;
  begin
    S.Read(Dummy, 2);
    {$IFDEF DEBUG} WriteLn('ate ', Dummy, ' vgl ', w); {$ENDIF}
  //if Dummy<>w then raise EFileCorrupt.Create('Illegal ID in procedure ImportWR1(var S:TStream)');
  end;

// GetInt reads an integer
  function GetOldInt:Int16; begin S.Read(Result, 2) end;

begin
// Import all "staticL"
  // Read the essential of A collection
   Eat(ImportCollID); Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);
  {$IFDEF DEBUG} WriteLn('Static lines :', Number); {$ENDIF}
  for i:=0 TO Number-1 do begin
    Eat(ImportLineID);
    B:=TLineAtom.Create; (B AS TLineAtom).ImportWR1(S);
    B.EditState:=0;
    BasicElements.Add(B);
  end;

// Import all "staticB"
   // Read the essential of A collection
   Eat(ImportCollID); Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);
  {$IFDEF DEBUG} WriteLn('Static curves :', Number); {$ENDIF}
  for i:=0 TO Number-1 do begin
    Eat(ImportCurveID);
    B:=TCSegAtom.Create; (B AS TCSegAtom).ImportWR1(S);
    B.EditState:=0;
    BasicElements.Add(B);
  end;

// Import all lines
   // Read the essential of A collection
   Eat(ImportCollID); Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);
  {$IFDEF DEBUG} WriteLn('Rail lines :', Number); {$ENDIF}
  for i:=0 TO Number-1 do begin
    Eat(ImportLineID);
    B:=TLineAtom.Create; (B AS TLineAtom).ImportWR1(S);
    B.EditState:=CAtomTrack or CAtomEdLk;
    BasicElements.Add(B);
  end;

// Import all curves
   // Read the essential of A collection
   Eat(ImportCollID); Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);
  {$IFDEF DEBUG} WriteLn('Rail curves :', Number); {$ENDIF}
  for i:=0 TO Number-1 do begin
    Eat(ImportCurveID);
    B:=TCSegAtom.Create; (B AS TCSegAtom).ImportWR1(S);
    B.EditState:=CAtomTrack or CAtomEdLk;
    BasicElements.Add(B);
  end;

// Import all cconnectors
   // Read A Byte WITH the Nr of the Last connection should prob. correspond
   // WITH Number
   S.Read(BNumber, 1);
   // Read the essential of A collection
   Eat(ImportCollID); Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);
  {$IFDEF DEBUG} WriteLn('connectors :', Number); {$ENDIF}
  for i:=0 TO Number-1 do begin
    Eat(ImportConnectorID);
    B:=TCnctAtom.Create;
    B.EditState:=CAtomEdLk;
    (B AS TCnctAtom).ImportWR1(S); (B AS TCnctAtom).Nr:=i;
    BasicElements.Add(B);
  end;

// Import element inforlmation
  S.Read(ElInfo, SizeOf(ElInfo));
  if ElInfo.TheType=UebergangsStueck then begin
    S.Read(Uebergang, SizeOf(Uebergang));
    StrCopy(Info.Uebergang, Uebergang);
  end;
  WITH ElInfo do begin
    StrCopy(Info.Name, Name);
    StrCopy(Info.Remark, Bemerkung);
    Info.TheType:=TheType;
    Info.TheRadius:=TheRadius;
    Info.TheLength:=TheLaenge;
    Info.TheAngle:=TheWinkel;
  end;
  {$IFDEF DEBUG}
    WITH ElInfo do begin
      WriteLn('Name   :', Name);
      WriteLn('Remark :', Bemerkung);
      WriteLn('Radius :', TheRadius:2:2);
      WriteLn('Length :', TheLaenge:2:2);
      WriteLn('Angle  :', TheWinkel:2:2);
      case TheType of
        Gerade : WriteLn('Gerade');
        Kurve  : WriteLn('Kurve');
        UebergangsStueck:WriteLn('Uebergang zu:',Uebergang);
      else WriteLn('Unbekannt')
      end
    end;
  {$ENDIF}
end;

{$IFDEF XMAS}
  procedure TElement.ExportWR1(var S:TStream);
  var Number    : integer;
      i         : integer;
      B         : TBasicAtom;
      BNumber   : Byte;
      Uebergang : array[0..25] of CHAR;
      ElInfo    : TImportElementInfo;
      TempList  : TList;

    procedure SpitOut(w:Word);
    begin
      S.Write(w, 2);
    end;

    procedure PutOldInt(X:Int16);begin S.Write(X, 2) end;

  begin
  // Export all "staticL"
    // Read the essential of A collection
    TempList:=TList.Create;
    for i:=0 TO BasicElements.Count-1 do begin
      B:=BasicElements.Items[i];
      if B is TLineAtom then begin
        if (B.EditState and CAtomTrack)=0 then TempList.Add(B);
      end;
    end;
    Number:=TempList.Count;
    SpitOut(ImportCollID); PutOldInt(Number); SpitOut(Number); SpitOut(5);
    for i:=0 TO Number-1 do begin
      SpitOut(ImportLineID);
      B:=TempList.Items[i];
      (B AS TLineAtom).ExportWR1(S);
    end;
    for i:=Number-1 downto 0 do TempList.Delete(i);
    TempList.Free;

  // Export all "staticB"
    TempList:=TList.Create;
    for i:=0 TO BasicElements.Count-1 do begin
      B:=BasicElements.Items[i];
      if B is TCSegAtom then begin
        if (B.EditState and CAtomTrack)=0 then TempList.Add(B);
      end;
    end;
    Number:=TempList.Count;
    SpitOut(ImportCollID); PutOldInt(Number); SpitOut(Number); SpitOut(5);
    for i:=0 TO Number-1 do begin
      SpitOut(ImportCurveID);
      B:=TempList.Items[i];
      (B AS TCSegAtom).ExportWR1(S);
    end;
    for i:=Number-1 downto 0 do TempList.Delete(i);
    TempList.Free;

  // Export all lines
    TempList:=TList.Create;
    for i:=0 TO BasicElements.Count-1 do begin
      B:=BasicElements.Items[i];
      if B is TLineAtom then begin
        if (B.EditState and CAtomTrack)>0 then TempList.Add(B);
      end;
    end;
    Number:=TempList.Count;
    SpitOut(ImportCollID); PutOldInt(Number); SpitOut(Number); SpitOut(5);
    for i:=0 TO Number-1 do begin
      SpitOut(ImportLineID);
      B:=TempList.Items[i];
      (B AS TLineAtom).ExportWR1(S);
    end;
    for i:=Number-1 downto 0 do TempList.Delete(i);
    TempList.Free;


  // Export all curves
    TempList:=TList.Create;
    for i:=0 TO BasicElements.Count-1 do begin
      B:=BasicElements.Items[i];
      if B is TCSegAtom then begin
        if (B.EditState and CAtomTrack)>0 then TempList.Add(B);
      end;
    end;
    Number:=TempList.Count;
    SpitOut(ImportCollID); PutOldInt(Number); SpitOut(Number); SpitOut(5);
    for i:=0 TO Number-1 do begin
      SpitOut(ImportCurveID);
      B:=TempList.Items[i];
      (B AS TCSegAtom).ExportWR1(S);
    end;
    for i:=Number-1 downto 0 do TempList.Delete(i);
    TempList.Free;

    TempList:=TList.Create;
    for i:=0 TO BasicElements.Count-1 do begin
      B:=BasicElements.Items[i];
      if B is TCnctAtom then begin
        TempList.Add(B);
      end;
    end;

    Number:=TempList.Count;
    BNumber:=Number;
    S.Write(BNumber, 1);
    SpitOut(ImportCollID); PutOldInt(Number); SpitOut(Number); SpitOut(5);
    for i:=0 TO Number-1 do begin
      SpitOut(ImportConnectorID);
      B:=TempList.Items[i];
      (B AS TCnctAtom).ExportWR1(S);
    end;

  // Export element inforlmation
    WITH ElInfo do begin
      StrCopy(Name, Info.Name);
      StrCopy(Bemerkung, Info.Remark);
      TheType:=Info.TheType;
      TheRadius:=Info.TheRadius;
      TheLaenge:=Info.TheLength;
      TheWinkel:=Info.TheAngle;
      StrCopy(Uebergang, Info.Uebergang);
    end;
    S.Write(ElInfo, SizeOf(ElInfo));
    if ElInfo.TheType=UebergangsStueck then S.Write(Uebergang, SizeOf(Uebergang));
  end;
{$ENDIF}

procedure TElement.UseGravity(var P:TDoublePoint; var A:Double; Radius:Double);
var APoint    : TDoublePoint;
    ADist     : Double;
    AAngle    : Double;
begin
  Radius:=SQR(Radius); // Let'S save A SQRT
  ADist:=GetNearestPointAndAngle(P, AAngle, APoint);
  if ADist<=Radius then begin
    A:=AAngle; P:=APoint;
  end;
end;

function TElement.GetNearestPointAndAngle(Point:TDoublePoint; var Angle:Double; var ResultPoint:TDoublePoint):Double;
var
    i            : longint;
    MinDistPoint : TDoublePoint;
    MinDist      : Double;
    MinDistAngle : Double;
    TempDist     : Double;
    TempAngle    : Double;
    TempPoint    : TDoublePoint;
begin
  if BasicElements.Count>0 then begin
    MinDist:=TBasicAtom(BasicElements.Items[0]).GetNearestPointAndAngle(Point, MinDistAngle, MinDistPoint);
  end else begin
    Angle:=0;
    Result:=0;
    ResultPoint:=Point;
    Exit;
  end;

  for i:=0 TO BasicElements.Count-1 do begin
    TempDist:=TBasicAtom(BasicElements.Items[i]).GetNearestPointAndAngle(Point, TempAngle, TempPoint);
    if TempDist<MinDist then begin
      MinDist:=TempDist; MinDistAngle:=TempAngle; MinDistPoint:=TempPoint;
    end;
  end;
  Angle:=MinDistAngle;
  ResultPoint:=MinDistPoint;
  Result:=MinDist;
end;

function TElement.GetNrOfConnectors:integer;
var i:integer;
begin
  Result:=0;
  for i:=0 TO BasicElements.Count-1 do begin
    if TBasicAtom(BasicElements.Items[i]) is TCnctAtom then Inc(Result);
  end;
end;

function TElement.GetNextCnctID:integer;
var i:integer;
    DummyPos:TDoublePoint;
    DummyAngle:Double;
begin
  Result:=0;
// not very efficient but this is not needed here
  for i:=0 TO BasicElements.Count+1 do begin
     if not GetConnectorPos(i, DummyPos, DummyAngle) then begin
       Result:=i;
       Exit;
     end;
  end;
end;

function TElement.GetConnectorPos(Nr:integer; var P:TDoublePoint; var Angle:Double):boolean;
var i:integer;
begin
  Result:=FALSE;
  for i:=0 TO BasicElements.Count-1 do begin
    if TBasicAtom(BasicElements.Items[i]) is TCnctAtom then begin
      if TCnctAtom(BasicElements.Items[i]).Nr = Nr then begin
        P:=TCnctAtom(BasicElements.Items[i]).P;
        Angle:=TCnctAtom(BasicElements.Items[i]).w;
        Result:=True;
        Exit;
      end;
    end;
  end;
end;

function TElement.GetConnectorWorldPos(Nr:integer; XS, YS, Winkel:Double; var P:TDoublePoint; var Angle:Double):boolean;
var i:integer;
    Cw, Sw:Extended;
    Pu:TDoublePoint;
begin
  Result:=FALSE;
  SinCos(Winkel, Sw, Cw);
  for i:=0 TO BasicElements.Count-1 do begin
    if TBasicAtom(BasicElements.Items[i]) is TCnctAtom then begin
      if TCnctAtom(BasicElements.Items[i]).Nr = Nr then begin
        Pu:=TCnctAtom(BasicElements.Items[i]).P;
        P.X:=(XS+Pu.X*Cw-Pu.Y*Sw);
        P.Y:=(YS+Pu.X*Sw+Pu.Y*Cw);
        Angle:=TCnctAtom(BasicElements.Items[i]).w+Winkel;
        FixAngle(Angle);
        Result:=True;
        Exit;
      end;
    end;
  end;
end;


destructor TElement.Free;
var i:integer;
begin
  for i:=BasicElements.Count-1 downto 0 do begin
    TBasicAtom(BasicElements.Items[i]).Free;
    BasicElements.Delete(i);
  end;
  BasicElements.Free;
end;

end.
