(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit Globals;

interface

USES Windows, Messages, Classes, Forms, Graphics, ComCtrls, Registry, Modules, SysUtils, RailyTypes,
     Math, GeneralConstants, AppTranslator, ImportTypes;

const WM_LoadFile = WM_User + 1;
     

var GlobalProgressBar  : TProgressBar;
    GlobalAppTranslator: TApptranslator;
    GlobalModules      : TModuleCollection;
    GlobalModulePath   : string;
    GlobalWorkPath     : string;
    GlobalExportPath   : string;
    GlobalElEdGridSize : DOUBLE;
    GlobalElEdGridShow : boolean;
    GlobalElEdGravity  : boolean;
    GlobalElEdGRange   : DOUBLE;
    GlobalElEdExact    : boolean;
    GlobalElEdPrec     : longint;
    GlobalElEdDouble   : BYTE;
    GlobalElEdMaxUndo  : integer;
    GlobalOverview     : boolean;
    ElEdChanged        : boolean;
    GlobalPathBase     : string;
    GlobalAvailLangList: TStringList;
    GlobalCurrentLang  : string;
    GlobalFXFadeoutTime: integer;
    GlobalClipboard    : TStream;
    GlobalDistTol      : DOUBLE;
    GlobalAngleTol     : DOUBLE;
    GlobalTop,
    GlobalLeft,
    GlobalWidth,
    GlobalHeight       : integer;
    GlobalElEdTop,
    GlobalElEdLeft,
    GlobalElEdWidth,
    GlobalElEdHeight   : integer;
    GlobalElEdTransPoly: boolean;
    GlobalCDROMDrive   : string;

//  Plan
    GlobalPlanCheapGray: boolean;
    GlobalPlanSingle   : boolean;
    GlobalPlanDouble   : boolean;
    GlobalPlanBettung  : boolean;
    GlobalPlanExact    : boolean;
    GlobalPlanPrec     : longint;
    GlobalFlexLegalRad : DOUBLE;
    GlobalFlexRadiusChk: boolean;
    GlobalPlanSpeedDraw: boolean;
    GlobalPlanGridSize : DOUBLE;
    GlobalPlanShowGrid : boolean;
    GlobalPlanPrintGrid: boolean;
    GlobalPlanPrintTable:boolean;
    GlobalTableSizeNrs : boolean;
    GlobalPlanIncTable : boolean;
    GlobalRailNrHeight : integer;
    GlobalPlanShowNames: boolean;
    GlobalPlanUseLayers: boolean;
    GlobalPlanGrayLayer: boolean;
    GlobalPlanUserTXT  : boolean;
    GlobalUserTXTHeight: integer;
    GlobalBezierPoints : integer;
    GlobalGravityRange : DOUBLE;
    GlobalCheckDiffMod : boolean;
    GlobalTransRemark  : boolean;
    GlobalDeltaAngle   : DOUBLE;
    GlobalPlanShowArrow: boolean;
    GlobalPlanSeparate : boolean;
    GlobalPlanTransPoly: boolean;


// Inventory
    GlobalInvHeight    : integer;
    GlobalInvFontHeight: integer;
    GlobalInvPictures  : boolean;
    GlobalInvLM        : integer;
    GlobalInvTM        : integer;
    GlobalInvRM        : integer;
    GlobalInvBM        : integer;
    GlobalInvHeader    : boolean;

    ParentElements     : TParentElements;
    ParentElements_ChooseBTN     : procedure (X:TImportSchienenTyp) of object;
    ParentElements_ChooseMode    : procedure (X:integer) of object;
    ParentElements_SpecialMode   : procedure (X:boolean) of object;
    ParentElements_IsClosing     : procedure of object;
    ParentElements_InfoFloaterVisible: procedure (X:boolean) of object;
    GlobalScrollThick  : integer;

// Printing
    GlobalPrintPNRSize : integer;
    GlobalPrintShowPP  : boolean;
    GlobalPrintBW      : boolean;

// Common procedures
    function CommonMakeString(X:DOUBLE):string;
    function CommonMakeDouble(X:string):DOUBLE;
    procedure CommonFixAngle(var w:DOUBLE); register;
    procedure ResetProfile;
    function FixedAngle(w:DOUBLE):DOUBLE; register;
    function EchtSchnitt(A, B, C, D:TDoublePoint; var R:TDoublePoint):boolean;
    function Angle(A, B, C, D: TDoublePoint):DOUBLE;
    function GlobalRotatePoint(P, pivot:TDoublePoint; aw:DOUBLE):TDoublePoint;
    function TransColor(C1, C2:TColor; P:Single):TColor; register;
    function CalcDist(X, Y:TDoublePoint):DOUBLE; register;
    procedure InitSearch(var P, PMI:array of TPoint; var PM:TPoint; SearchSize:longint);
    function LineSelected(PL, PA: TPoint; PMI, P:array of TPoint): boolean; register;
    function Middle(P1, P2:TDoublePoint):TDoublePoint;
//
    function TransStr(X:string):string;
    function Confirm(X:string):boolean;
    procedure Inform(X:string);
    procedure Warn(X:string);
    function ConfirmAbort(X:string):integer;
    procedure StringToStream(St:string; S:TStream);
    function StringFromStream(S:TStream):string;
    procedure FFont(X:TCanvas; Name:string; Height, Boldness:integer);
    function MyAng(X, Y:DOUBLE):DOUBLE;
    function XMirrorPoint(Mirror:DOUBLE; P:TDoublePoint):TDoublePoint;
    function YMirrorPoint(Mirror:DOUBLE; P:TDoublePoint):TDoublePoint;
    function IsReadOnly(Name:string):boolean;

implementation

function IsReadOnly(Name:string):boolean;
var
  SearchRec : TSearchRec;
begin
  Result:=FALSE;
  if FindFirst(Name, faAnyFile, SearchRec)=0 then Result:=(SearchRec.Attr and faReadOnly) >0;
  FindClose(SearchRec);
end;

function XMirrorPoint(Mirror:DOUBLE; P:TDoublePoint):TDoublePoint;
begin
  Result:=P;
  Result.X:=Mirror-(P.X-Mirror);
end;

function YMirrorPoint(Mirror:DOUBLE; P:TDoublePoint):TDoublePoint;
begin
  Result:=P;
  Result.Y:=Mirror-(P.Y-Mirror);
end;

function MyAng(X, Y:DOUBLE):DOUBLE;
begin
  try
     Result:=Arctan2(Y, X);
  except
     Result:=0;
  end;
end;

procedure FFont(X:TCanvas; Name:string; Height, Boldness:integer);
var NF:integer;
begin
  NF:=CreateFont(-Height, 0, 0, 0, Boldness, 0, 0, 0, 1, OUT_TT_ONLY_PRECIS, $10, 2, DEFAULT_PITCH or FF_SWISS, PCHAR(Name));
  DeleteObject(SelectObject(X.Handle, NF));
end;

procedure StringToStream(St:string; S:TStream);
var i, L:integer;
begin
  L:=Length(St);
  S.Write(L, SizeOf(i));
  for i:=1 TO L do S.Write(St[i], SizeOf(St[i]));
end;

function StringFromStream(S:TStream):string;
var i, L:integer;
    C:Char;
begin
  Result:='';
  try
    if S.Position>=S.Size then raise Exception.Create('FileError');
    S.Read(L, SizeOf(i));
    for i:=1 TO L do begin
      if S.Position>=S.Size then raise Exception.Create('FileError');
      S.Read(C, SizeOf(C));
      Result:=Result+C;
    end;
  except Result:=''; end;  
end;

function TransStr(X:string):string;
begin
  Result:=GlobalAppTranslator.GetStr(X);
end;

function Confirm(X:string):boolean;
begin
  Result:=Application.MessageBOX(PCHAR(X), PCHAR(TransStr('[Confirmation]')), MB_ICONQuestion or MB_YesNo)=IDYes;
end;

procedure Inform(X:string);
begin
  Application.MessageBOX(PCHAR(X), PCHAR(TransStr('[Information]')), MB_ICONInformation or MB_Ok);
end;

procedure Warn(X:string);
begin
  Application.MessageBOX(PCHAR(X), PCHAR(TransStr('[Warning]')), MB_ICONExclamation or MB_Ok);
end;

function ConfirmAbort(X:string):integer;
begin
  Result:=Application.MessageBOX(PCHAR(X), PCHAR(TransStr('[Confirmation]')), MB_ICONQuestion or MB_YesNoCancel);
end;

function CalcDist(X, Y:TDoublePoint):DOUBLE; register;
begin
  try
    Result:=Hypot(ABS(X.X-Y.X), ABS(X.Y-Y.Y));
  except
    Result:=10E30;
  end;
end;

//Gemss Robert Sedgewick Seiten 402-407
function iCCW(p0,P1,P2:TPoint):integer; register;
var dx1, dx2, dy1, dy2: longint;
begin
  dx1:=P1.X-p0.X; dy1:=P1.Y-p0.Y;
  dx2:=P2.X-p0.X; dy2:=P2.Y-p0.Y;
  if dx1*dy2>dy1*dx2 then Result:=1
   else if dx1*dy2<dy1*dx2 then begin
     Result:=-1;
   end else begin
     if (dx1*dx2<0) or (dy1*dy2<0) then Result:=-1 else
       if (dx1*dx1 + dy1*dy1) >= (dx2*dx2 + dy2*dy2) then Result:=0 else Result:=-1;
   end;
end;

function iIntersect(A, B, C, D:TPoint):boolean; register;
begin
  Result:=(iCCW(A, B, C)*iCCW(A, B, D)<=0) and
          (iCCW(C, D, A)*iCCW(C, D, B)<=0);
end;

function LineSelected(PL, PA: TPoint; PMI, P:array of TPoint): boolean; register;
var i:integer;
begin
  Result:=FALSE;
  for i:=0 TO 3 do if iIntersect(PL, PA, PMI[i], P[i]) then begin
    Result:=True;
    break;
  end;
  // if one point is within the rectangle and no intersection is detected the other is inside, too
  // and then the line is selected
  if not Result then  Result:=((PL.X>PMI[0].X) and (PL.Y>PMI[0].Y)) and ((PL.X<P[3].X) and (PL.Y<P[3].Y));
end;

procedure InitSearch(var P, PMI:array of TPoint; var PM:TPoint; SearchSize:longint);
begin
  PMI[0].Y:=PM.Y-SearchSize; P[0].Y:=PM.Y+SearchSize;
  PMI[0].X:=PM.X-SearchSize; P[0].X:=PM.X-SearchSize;
  PMI[1].Y:=PM.Y-SearchSize; P[1].Y:=PM.Y+SearchSize;
  PMI[1].X:=PM.X+SearchSize; P[1].X:=PM.X+SearchSize;
  PMI[2].Y:=PM.Y-SearchSize; P[2].Y:=PM.Y-SearchSize;
  PMI[2].X:=PM.X-SearchSize; P[2].X:=PM.X+SearchSize;
  PMI[3].Y:=PM.Y+SearchSize; P[3].Y:=PM.Y+SearchSize;
  PMI[3].X:=PM.X-SearchSize; P[3].X:=PM.X+SearchSize;
end;

function Middle(P1, P2:TDoublePoint):TDoublePoint;
begin
  Result.X:=0.5*(P1.X+P2.X);
  Result.Y:=0.5*(P1.Y+P2.Y);
end;


// rotates A point around another
function GlobalRotatePoint(P, pivot:TDoublePoint; aw:DOUBLE):TDoublePoint;
var cw, sw:Extended;
begin
  P.X:=P.X-pivot.X; P.Y:=P.Y-pivot.Y;
  SinCos(aw, sw, cw);
  Result.X:=(P.X*cw-P.Y*sw)+pivot.X;
  Result.Y:=(P.X*sw+P.Y*cw)+pivot.Y;
end;

function EchtSchnitt(A, B, C, D:TDoublePoint; var R:TDoublePoint):boolean;
var w, v: TDoublePoint;
    m:DOUBLE;
begin
  Result:=True;
  w.X:=B.X-A.X; w.Y:=B.Y-A.Y;
  v.X:=D.X-C.X; v.Y:=D.Y-C.Y;
  if ABS((v.Y*w.X - v.X*w.Y))<16*10E-39 then begin
    Result:=FALSE;
    R:=A;
    Exit;
  end;
  m:=((A.Y-C.Y)*w.X + (C.X-A.X)*w.Y) /
      (v.Y*w.X - v.X*w.Y);
  R.X:=C.X+m*v.X;
  R.Y:=C.Y+m*v.Y;
end;

function Angle(A, B, C, D: TDoublePoint):DOUBLE; register;
var D1, D2:TDoublePoint;
    Alpha, Beta:DOUBLE;
begin
  D1.X:=B.X-A.X; D1.Y:=B.Y-A.Y;
  D2.X:=D.X-C.X; D2.Y:=D.Y-C.Y;
  Alpha:=D1.X*D2.X+D1.Y*D2.Y;
  Beta:=Hypot(D1.X, D1.Y)*Hypot(D2.X, D2.Y);
  try
    Result:=ArcCos(Alpha/Beta);
  except
    Result:=0;
  end;
end;

function CommonMakeDouble(X:string):DOUBLE;
var C:integer;
begin
  Val(X, Result, C);
  if C<>0 then Result:=1;
end;

function CommonMakeString(X:DOUBLE):string;
begin
  Str(X:2:9, Result);
  Result:=TrimLeft(Result);
end;

procedure CommonFixAngle(var w:DOUBLE); register;
begin
  if w<0 then w:=w+(2*Pi*trunc(ABS(w)/(2*Pi)+1));
  if w>0 then w:=w-(2*Pi*trunc(w/(2*Pi)));
  if ABS(w-2*Pi)<0.0000001 then w:=0;
end;

function FixedAngle(w:DOUBLE):DOUBLE; register;
begin
  CommonFixAngle(w);
  Result:=w;
end;

//******************************************************************************
// procedures for color manipulations
//******************************************************************************
procedure GetRGB(X:TColor; var R, G, B:integer); register;
var L:longint;
begin
  L:=ColorToRGB(X);
  R:=(L and $000000FF);
  G:=(L and $0000FF00) SHR 8;
  B:=(L and $00FF0000) SHR 16;
end;

function SetRGB(R, G, B:integer):TColor; register;
begin
  Result:=0;
  Result:=Result or (R and $FF);
  Result:=Result or (G and $FF) SHL 8;
  Result:=Result or (B and $FF) SHL 16;
end;

function TransColor(C1, C2:TColor; P:Single):TColor; register;
var R1, G1, B1, R2, G2, B2:integer;
begin
  GetRGB(C1, R1, G1, B1);
  GetRGB(C2, R2, G2, B2);
  Result:=SetRGB(Round(R1+(R2-R1)*P), Round(G1+(G2-G1)*P), Round(B1+(B2-B1)*P));
end;


procedure ReadProfile;
var Registry: TRegistry;
begin
  GlobalModulePath:=ExtractFilePath(Application.EXEName);
  GlobalPathBase:=ExtractFilePath(Application.EXEName);
  GlobalFXFadeoutTime:=20;

  Registry:=TRegistry.Create;

// Registry for main program
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0', True);
  GlobalTop:=1;     GlobalLeft:=1;
  GlobalWidth:=610; GlobalHeight:=402;
  GlobalModulePath:=ExtractFilePath(Application.EXEName);
  GlobalWorkPath:=ExtractFilePath(Application.EXEName);
  GlobalExportPath:=ExtractFilePath(Application.EXEName);
  GlobalCurrentLang:='Deutsch';
  // Position
  if Registry.ValueExists('Top') then GlobalTop:=Registry.ReadInteger('Top');
  if Registry.ValueExists('Left') then GlobalLeft:=Registry.ReadInteger('Left');
  if Registry.ValueExists('Width') then GlobalWidth:=Registry.ReadInteger('Width');
  if Registry.ValueExists('Height') then GlobalHeight:=Registry.ReadInteger('Height');
  if Registry.ValueExists('Language') then GlobalCurrentLang:=Registry.ReadString('Language');
  if Registry.ValueExists('ModulePath') then GlobalModulePath:=Registry.ReadString('ModulePath');
  if Registry.ValueExists('WorkingPath') then GlobalWorkPath:=Registry.ReadString('WorkingPath');
  if Registry.ValueExists('ExportPath') then GlobalExportPath:=Registry.ReadString('ExportPath');
  if Registry.ValueExists('CDROMDrive') then GlobalCDROMDrive:=Registry.ReadString('CDROMDrive');

  Registry.CloseKey;

// Registry for Plan editor
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0\PlanEd', True);
  GlobalPlanCheapGray:= GetDeviceCaps(GetDC(0),BITSPIXEL)<9;
  GlobalPlanSingle   := FALSE;
  GlobalPlanDouble   := True;
  GlobalPlanBettung  := FALSE;
  GlobalPlanExact:=True;
  GlobalPlanPrec    := 128;
  GlobalFlexLegalRad:=360;
  GlobalFlexRadiusChk:=True;
  GlobalPlanSpeedDraw:=True;
  GlobalPlanGridSize:=50;
  GlobalPlanShowGrid:=FALSE;
  GlobalPlanPrintGrid:=FALSE;
  GlobalPlanPrintTable:=True;
  GlobalTableSizeNrs  := True;
  GlobalPlanIncTable  := True;
  GlobalRailNrHeight  := 20;
  GlobalUserTXTHeight := 30;
  GlobalPlanShowNames := True;
  GlobalPlanUserTXT   := True;
  GlobalPlanUseLayers := True;
  GlobalPlanGrayLayer := True;
  GlobalBezierPoints  := 64;
  GlobalGravityRange  := 100;
  GlobalCheckDiffMod  := True;
  GlobalPlanShowArrow := FALSE;
  GlobalPlanSeparate  := True;
  GlobalTransRemark   := True;
  GlobalDeltaAngle    := 5;
  GlobalDistTol       := 10;
  GlobalAngleTol      := Pi/180;
  GlobalOverview      := True;
  GlobalPlanTransPoly := FALSE;

  if Registry.ValueExists('CheapGray') then GlobalPlanCheapGray:=Registry.ReadBool('CheapGray');
  if Registry.ValueExists('SingleLine') then GlobalPlanSingle:=Registry.ReadBool('SingleLine');
  if Registry.ValueExists('DoubleLine') then GlobalPlanDouble:=Registry.ReadBool('DoubleLine');
  if Registry.ValueExists('ElementSizeLine') then GlobalPlanBettung:=Registry.ReadBool('ElementSizeLine');
  if Registry.ValueExists('Exact') then GlobalPlanExact:=Registry.ReadBool('Exact');
  if Registry.ValueExists('Precision') then GlobalPlanPrec:=Registry.ReadInteger('Precision');
  if Registry.ValueExists('MinRadius') then GlobalFlexLegalRad:=Registry.ReadFloat('MinRadius');
  if Registry.ValueExists('CheckRadius') then GlobalFlexRadiusChk:=Registry.ReadBool('CheckRadius');

  if Registry.ValueExists('SpeedDraw') then GlobalPlanSpeedDraw:=Registry.ReadBool('SpeedDraw');
  if Registry.ValueExists('GridSize') then GlobalPlanGridSize:=Registry.ReadFloat('GridSize');
  if Registry.ValueExists('ShowGrid') then GlobalPlanShowGrid:=Registry.ReadBool('ShowGrid');
  if Registry.ValueExists('PrintGrid') then GlobalPlanPrintGrid:=Registry.ReadBool('PrintGrid');
  if Registry.ValueExists('PrintTable') then GlobalPlanPrintTable:=Registry.ReadBool('PrintTable');
  if Registry.ValueExists('ShowTableSizes') then GlobalTableSizeNrs:=Registry.ReadBool('ShowTableSizes');
  if Registry.ValueExists('IncludeTable') then GlobalPlanIncTable:=Registry.ReadBool('IncludeTable');
  if Registry.ValueExists('ElementTextHeight') then GlobalRailNrHeight:=Registry.ReadInteger('ElementTextHeight');
  if Registry.ValueExists('UserTextHeight') then GlobalUserTXTHeight:=Registry.ReadInteger('UserTextHeight');
  if Registry.ValueExists('ShowRailNames') then GlobalPlanShowNames:=Registry.ReadBool('ShowRailNames');
  if Registry.ValueExists('UserText') then GlobalPlanUserTXT:=Registry.ReadBool('UserText');
  if Registry.ValueExists('UseLayers') then GlobalPlanUseLayers:=Registry.ReadBool('UseLayers');
  if Registry.ValueExists('GrayLayers') then GlobalPlanGrayLayer:=Registry.ReadBool('GrayLayers');
  if Registry.ValueExists('BezierPoints') then GlobalBezierPoints:=Registry.ReadInteger('BezierPoints');
  if Registry.ValueExists('GravityRange') then GlobalGravityRange:=Registry.ReadFloat('GravityRange');
  if Registry.ValueExists('CheckDifferentModules') then GlobalCheckDiffMod:=Registry.ReadBool('CheckDifferentModules');
  if Registry.ValueExists('CnctArrows') then GlobalPlanShowArrow:=Registry.ReadBool('CnctArrows');
  if Registry.ValueExists('CnctSeparate') then GlobalPlanSeparate:=Registry.ReadBool('CnctSeparate');
  if Registry.ValueExists('UserDeltaAngle') then GlobalDeltaAngle:=Registry.ReadFloat('UserDeltaAngle');
  if Registry.ValueExists('DistTolerance') then GlobalDistTol:=Registry.ReadFloat('DistTolerance');
  if Registry.ValueExists('AngleTolerance') then GlobalAngleTol:=Registry.ReadFloat('AngleTolerance');
  if Registry.ValueExists('AutoOverview') then GlobalOverview:=Registry.ReadBool('AutoOverview');
  if Registry.ValueExists('TransparentPolygon') then GlobalPlanTransPoly:=Registry.ReadBool('TransparentPolygon');


  Registry.CloseKey;


// Registry for Element editor
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0\ElEd', True);
  GlobalElEdTop:=1;     GlobalElEdLeft:=1;
  GlobalElEdWidth:=610; GlobalElEdHeight:=402;
  GlobalElEdGravity := True; GlobalElEdGRange  := 10;
  GlobalElEdExact   := True; GlobalElEdDouble  := CDrawInner;
  GlobalElEdPrec    := 128;  GlobalElEdMaxUndo := 100;
  GlobalElEdGridSize:=10;
  GlobalElEdGridShow:=True;
  GlobalElEdTransPoly:=FALSE;
  // Position
  if Registry.ValueExists('Top') then GlobalElEdTop:=Registry.ReadInteger('Top');
  if Registry.ValueExists('Left') then GlobalElEdLeft:=Registry.ReadInteger('Left');
  if Registry.ValueExists('Width') then GlobalElEdWidth:=Registry.ReadInteger('Width');
  if Registry.ValueExists('Height') then GlobalElEdHeight:=Registry.ReadInteger('Height');
  // Internal
  if Registry.ValueExists('Precision') then GlobalElEdPrec:=Registry.ReadInteger('Precision');
  if Registry.ValueExists('MaxUndo') then GlobalElEdMaxUndo:=Registry.ReadInteger('MaxUndo');
  if Registry.ValueExists('GravityRange') then GlobalElEdGRange :=Registry.ReadFloat('GravityRange');
  if Registry.ValueExists('GridSize') then GlobalElEdGridSize :=Registry.ReadFloat('GridSize');
  if Registry.ValueExists('Exact') then GlobalElEdExact:=Registry.ReadBool('Exact');
  if Registry.ValueExists('Gravity') then GlobalElEdGravity:=Registry.ReadBool('Gravity');
  if Registry.ValueExists('ShowGrid') then GlobalElEdGridShow:=Registry.ReadBool('ShowGrid');
  if Registry.ValueExists('TrackMode') then GlobalElEdDouble:=BYTE(Registry.ReadInteger('TrackMode') MOD 255);
  if Registry.ValueExists('TransparentPolygon') then GlobalElEdTransPoly:=Registry.ReadBool('TransparentPolygon');

  Registry.CloseKey;

// Registry for Inventory
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0\Inventory', True);
  GlobalInvLM        := 10;  GlobalInvTM        := 10;
  GlobalInvRM        := 5;   GlobalInvBM        := 5;
  GlobalInvFontHeight:= 30;  GlobalInvHeight    := 30;
  GlobalInvHeader    := True;GlobalInvPictures  := True;
  if Registry.ValueExists('Pictures') then GlobalInvPictures:=Registry.ReadBool('Pictures');
  if Registry.ValueExists('LeftMarg') then GlobalInvLM:=Registry.ReadInteger('LeftMarg');
  if Registry.ValueExists('TopMarg') then GlobalInvTM:=Registry.ReadInteger('TopMarg');
  if Registry.ValueExists('RightMarg') then GlobalInvRM:=Registry.ReadInteger('RightMarg');
  if Registry.ValueExists('BottomMarg') then GlobalInvBM:=Registry.ReadInteger('BottomMarg');
  if Registry.ValueExists('LineHeight') then GlobalInvHeight:=Registry.ReadInteger('LineHeight');
  if Registry.ValueExists('FontHeight') then GlobalInvFontHeight:=Registry.ReadInteger('FontHeight');
  if Registry.ValueExists('Header') then GlobalInvHeader:=Registry.ReadBool('Header');
  Registry.CloseKey;

// Registry for Printing
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0\Printing', True);
  GlobalPrintPNRSize := 100;
  GlobalPrintShowPP  := True;
  GlobalPrintBW:=FALSE;
  if Registry.ValueExists('PgNrHeight') then GlobalPrintPNRSize:=Registry.ReadInteger('PgNrHeight');
  if Registry.ValueExists('ShowPrintingP') then GlobalPrintShowPP:=Registry.ReadBool('ShowPrintingP');
  if Registry.ValueExists('BlackWhite') then GlobalPrintBW:=Registry.ReadBool('BlackWhite');

  Registry.CloseKey;


  Registry.Free;
end;

procedure WriteProfile;
var Registry: TRegistry;
begin
  Registry:=TRegistry.Create;
// Registry for main program
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0', True);
  Registry.WriteInteger('Top', GlobalTop);
  Registry.WriteInteger('Left', GlobalLeft);
  Registry.WriteInteger('Width', GlobalWidth);
  Registry.WriteInteger('Height', GlobalHeight);
  Registry.WriteString('Language', GlobalCurrentLang);
  Registry.WriteString('ModulePath', GlobalModulePath);
  Registry.WriteString('WorkingPath', GlobalWorkPath);
  Registry.WriteString('ExportPath', GlobalExportPath);
  Registry.WriteString('CDROMDrive', GlobalCDROMDrive);
  Registry.CloseKey;

// Registry for Plan editor
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0\PlanEd', True);
  Registry.WriteBool('CheapGray',GlobalPlanCheapGray);
  Registry.WriteBool('SingleLine', GlobalPlanSingle);
  Registry.WriteBool('DoubleLine', GlobalPlanDouble);
  Registry.WriteBool('ElementSizeLine', GlobalPlanBettung);
  Registry.WriteBool('Exact', GlobalPlanExact);
  Registry.WriteInteger('Precision', GlobalPlanPrec);
  Registry.WriteFloat('MinRadius', GlobalFlexLegalRad);
  Registry.WriteBool('CheckRadius', GlobalFlexRadiusChk);
  Registry.WriteBool('SpeedDraw', GlobalPlanSpeedDraw);
  Registry.WriteFloat('GridSize', GlobalPlanGridSize);
  Registry.WriteBool('ShowGrid', GlobalPlanShowGrid);
  Registry.WriteBool('PrintGrid', GlobalPlanPrintGrid);
  Registry.WriteBool('PrintTable', GlobalPlanPrintTable);
  Registry.WriteBool('ShowTableSizes', GlobalTableSizeNrs);
  Registry.WriteBool('IncludeTable', GlobalPlanIncTable);
  Registry.WriteInteger('ElementTextHeight', GlobalRailNrHeight);
  Registry.WriteInteger('UserTextHeight', GlobalUserTXTHeight);
  Registry.WriteBool('ShowRailNames', GlobalPlanShowNames);
  Registry.WriteBool('UserText', GlobalPlanUserTXT);
  Registry.WriteBool('UseLayers', GlobalPlanUseLayers);
  Registry.WriteBool('GrayLayers', GlobalPlanGrayLayer);
  Registry.WriteInteger('BezierPoints', GlobalBezierPoints);
  Registry.WriteFloat('GravityRange', GlobalGravityRange);
  Registry.WriteBool('CheckDifferentModules', GlobalCheckDiffMod);
  Registry.WriteBool('CnctArrows', GlobalPlanShowArrow);
  Registry.WriteBool('CnctSeparate', GlobalPlanSeparate);
  Registry.WriteFloat('UserDeltaAngle', GlobalDeltaAngle);
  Registry.WriteFloat('DistTolerance', GlobalDistTol);
  Registry.WriteFloat('AngleTolerance', GlobalAngleTol);
  Registry.WriteBool('AutoOverview',GlobalOverview);
  Registry.WriteBool('TransparentPolygon', GlobalPlanTransPoly);
  Registry.CloseKey;

// Registry for Element editor
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0\ElEd', True);
  Registry.WriteInteger('Top', GlobalElEdTop);
  Registry.WriteInteger('Left', GlobalElEdLeft);
  Registry.WriteInteger('Width', GlobalElEdWidth);
  Registry.WriteInteger('Height', GlobalElEdHeight);
  // Internal
  Registry.WriteInteger('Precision', GlobalElEdPrec);
  Registry.WriteInteger('MaxUndo', GlobalElEdMaxUndo);
  Registry.WriteFloat('GravityRange', GlobalElEdGRange);
  Registry.WriteFloat('GridSize', GlobalElEdGridSize);
  Registry.WriteBool('Exact', GlobalElEdExact);
  Registry.WriteBool('Gravity', GlobalElEdGravity);
  Registry.WriteBool('ShowGrid', GlobalElEdGridShow);
  Registry.WriteInteger('TrackMode', GlobalElEdDouble);
  Registry.WriteBool('TransparentPolygon', GlobalElEdTransPoly);
  Registry.CloseKey;

// Registry for Inventory
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0\Inventory', True);
  Registry.WriteBool('Pictures', GlobalInvPictures);
  Registry.WriteInteger('LeftMarg', GlobalInvLM);
  Registry.WriteInteger('TopMarg', GlobalInvTM);
  Registry.WriteInteger('RightMarg', GlobalInvRM);
  Registry.WriteInteger('BottomMarg', GlobalInvBM);
  Registry.WriteInteger('LineHeight', GlobalInvHeight);
  Registry.WriteInteger('FontHeight', GlobalInvFontHeight);
  Registry.WriteBool('Header' ,GlobalInvHeader);
  Registry.CloseKey;

// Registry for Printing
  Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0\Printing', True);
  Registry.WriteInteger('PgNrHeight', GlobalPrintPNRSize);
  Registry.WriteBool('ShowPrintingP', GlobalPrintShowPP);
  Registry.WriteBool('BlackWhite', GlobalPrintBW);
  Registry.CloseKey;

  Registry.Free;
end;

procedure ResetProfile;
var Registry: TRegistry;
begin
  Registry:=TRegistry.Create;
  if Registry.KeyExists('SOFTWARE\ENIGON\Raily2.0') then begin
    Registry.DeleteKey('SOFTWARE\ENIGON\Raily2.0\ElEd');
    Registry.DeleteKey('SOFTWARE\ENIGON\Raily2.0\Inventory');
    Registry.DeleteKey('SOFTWARE\ENIGON\Raily2.0\Printing');
    Registry.DeleteKey('SOFTWARE\ENIGON\Raily2.0');
    Registry.OpenKey('SOFTWARE\ENIGON\Raily2.0', True);
    Registry.WriteString('CDROMDrive', GlobalCDROMDrive);
    Registry.CloseKey;
  end;
  Registry.Free;
  ReadProfile;
end;

procedure CleanUp; FAR;
begin
  GlobalModules.Free;
  GlobalAvailLangList.Free;
  GlobalClipboard.Free;
  WriteProfile;
end;

INITIALIZATION
  ElEdChanged:=FALSE;
  GlobalAvailLangList:=TStringList.Create;
  GlobalAvailLangList.duplicates:=dupIgnore;
  GlobalAvailLangList.Sorted:=True;
  GlobalScrollThick := 25;
  ReadProfile;
  GlobalModules:=TModuleCollection.Create;
  GlobalModules.ScanForModules(GlobalModulePath);
  GlobalClipboard:=TMemoryStream.Create;
FINALIZATION
  CleanUp;
end.
