(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com, DXF relevant parts added by Bernhard Egger
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit Instances;

interface

USES Classes, Graphics, SysUtils, RailyTypes, Elemente, Modules, Globals,
     Windows, ImportTypes, ImportConstants, GeneralConstants, Math,
     ULineExporter;

type
     TFlexAddition = class (TObject)
                  PolyLine1,
                  PolyLine2,
                  PolyLine3,
                  PolyLine4,
                  PolyLine5  : TList;

                  Illegals   : TBits;
                  Points     : TList;
                  A1, A2     : Double;
                  ModuleCache: TModule;
                  ElementCache:TElement;

                  constructor Create(X:TElement);
                  destructor Free;
                  function ToStream(var S:TStream):boolean;
                  function FromStream(var S:TStream):boolean;
                  procedure Draw(X: TCanvas; aX, aY, fX, fY: Double; DrawInfo:PDrawInfo);
                  procedure ExportLines(X: TLineExport; aX, aY, fX, fY: Double; DrawInfo:PDrawInfo);
                  function IsHit(ClX, ClY: longint; aX, aY, fX, fY: Double; DrawInfo:PDrawInfo):boolean;
                  procedure Calculate;
                  procedure FreePointList(var X:TList);
                  procedure Turnmove(dx, dy, DeltaAngle:Double; P1:TDoublePoint);
                  procedure CheckIn(var tUL, tLR:TDoublePoint);
                  function GetConnectorPos(Nr:integer; var P:TDoublePoint; var A:Double):boolean;
                  function GetLength:Double;
                  procedure AddPoint(Where:integer);
                  procedure DeletePoint(Which:integer);
     end;


     TInstance = class (TObject)
                   UL, LR       : TSinglePoint;
                   CheckUL,
                   CheckLR      : TSinglePoint;
                   Position     : TDoublePoint;
                   TextPos      : TDoublePoint;
                   Angle        : Double;
                   CacheElement : TElement;
                   ID           : longint;
                   Color        : TColor;
                   Status       : longint;
                   Layer        : longint;
                   ElementName  : array[0..20] of CHAR;
                   ModuleName   : array[0..25] of CHAR;
                   Connectors   : TList;
                   ModuleCache  : TModule;
                   FlexAddition : TFlexAddition;
                 private
                   procedure RemoveConnectors;

                 public
                   constructor Create;
                   destructor Free;
                   function AssignToElement(ModName, ElName:string): boolean;
                   function ToStream(var S:TStream):boolean; virtual;
                   function FromStream(var S:TStream):boolean; virtual;
                   function ImportWR1(var S:TStream): boolean;
                   function Recache:boolean;
                   procedure CheckIn;
                   procedure Draw(X: TCanvas; aX, aY, fX, fY: Double; Selected:boolean; DrawInfo:PDrawInfo); virtual;
                   procedure ExportLines(X: TLineExport; aX, aY, fX, fY: Double; Selected:boolean; DrawInfo:PDrawInfo); virtual;
                   function IsHit(ClX, ClY: longint; aX, aY, fX, fY: Double; DrawInfo:PDrawInfo):boolean; virtual;
                   function IsInRect(aUL, aDR: TDoublePoint):boolean;
                   function ConnectorCount:integer;
                   function GetConnector(Nr:integer; var C:TConnector):boolean;
                   function GetConnectorPos(Nr:integer; var P:TDoublePoint; var A:Double):boolean;

                   procedure SetEntireConnector(Nr:integer; C:TConnector);
                   procedure SetConnector(Nr, ToID:integer);
                   procedure SetConnectorHeight(Nr:integer; Height:Single);

                   function GetLength:Double;


                   procedure BreakApart;

                   procedure MakeFlex;
                   function IsFlex:boolean;

// Helper functions
                   procedure ResetNamePosition;
                 end;

implementation

constructor TInstance.Create;
begin
  CacheElement:=NIL; //Should already be initialized but who knows...

  Connectors:=TList.Create;

  Color:=clBlack; // Initial-Value
end;

function TInstance.ConnectorCount:integer;
begin
  Result:=Connectors.Count;
end;

function TInstance.GetConnector(Nr:integer; var C:TConnector):boolean;
begin
  Result:=FALSE;
  if (Nr>=0) and (Nr<Connectors.Count) then begin
    C:=PConnector(Connectors[Nr])^;
    Result:=TRUE;
  end;
end;

procedure TInstance.SetEntireConnector(Nr:integer; C:TConnector);
begin
  if (Nr>=0) and (Nr<Connectors.Count) then PConnector(Connectors[Nr])^:=C;
end;


procedure TInstance.SetConnector(Nr, ToID:integer);
begin
  if (Nr>=0) and (Nr<Connectors.Count) then PConnector(Connectors[Nr])^.Nr:=ToID;
end;

procedure TInstance.BreakApart;
var i:integer;
begin
  for i:=0 TO Connectors.Count-1 do PConnector(Connectors[i])^.Nr:=-1;
end;

procedure TInstance.SetConnectorHeight(Nr:integer; Height:Single);
begin
  if (Nr>=0) and (Nr<Connectors.Count) then PConnector(Connectors[Nr])^.Height:=Height;
end;

function TInstance.AssignToElement(ModName, ElName:string): boolean;
var M:TModule;
    i:integer;
begin
  Result:=FALSE;
  // if the module is already loaded then we get TRUE, nothing else happens.
  // if not GlobalModules will try TO load it. Only if this fails we get FALSE
  // and there is nothing TO do than TO fail, too.
  if GlobalModules.LoadModule(ModName) then
     if GlobalModules.GetModule(ModName, M) then begin
       ModuleCache:=M;
       if M.GetElement(ElName, CacheElement) then begin
         Result:=TRUE;
         RemoveConnectors;
         for i:=0 TO CacheElement.GetNrOfConnectors-1 do Connectors.Add(New(PConnector));
       end;
  end;
  StrPCopy(ModuleName, ModName);
  StrPCopy(ElementName, ElName);
end;

function TInstance.ToStream(var S:TStream):boolean;
var i, Number: longint;
    B        : boolean;
begin
  Result:=TRUE;
  try
    S.Write(Position   , SizeOf(Position));
    S.Write(Angle      , SizeOf(Angle));
    S.Write(ID         , SizeOf(ID));
    S.Write(Color      , SizeOf(Color));
    S.Write(Status     , SizeOf(Status));
    S.Write(ModuleName , SizeOf(ModuleName));
    S.Write(ElementName, SizeOf(ElementName));
    S.Write(TextPos   , SizeOf(TextPos));
    S.Write(Layer   , SizeOf(Layer));
    Number:=Connectors.Count;
    S.Write(Number, SizeOf(Number));
   for i:=0 TO Number-1 do begin
      S.Write(Connectors.Items[i]^, SizeOf(TConnector));
    end;
    B:=IsFlex;
    S.Write(B, SizeOf(B));
    if B then FlexAddition.ToStream(S);
  except
    Result:=FALSE;
  end;
end;

function TInstance.FromStream(var S:TStream):boolean;
var i, Number: longint;
    TempConn : TConnector;
    B        : boolean;
begin
  try
    S.Read(Position, SizeOf(Position));
    S.Read(Angle   , SizeOf(Angle));
    S.Read(ID      , SizeOf(ID));
    S.Read(Color   , SizeOf(Color));
    S.Read(Status  , SizeOf(Status));
    S.Read(ModuleName , SizeOf(ModuleName));
    S.Read(ElementName, SizeOf(ElementName));
    S.Read(TextPos   , SizeOf(TextPos));
    S.Read(Layer   , SizeOf(Layer));

    Result:=Recache;
    if not Result then raise Exception.Create('Element/module not found');
    CheckIn;
    S.Read(Number, SizeOf(Number));
    for i:=0 TO Number-1 do begin
      if S.Position>=S.Size then raise Exception.Create('');
      S.Read(TempConn, SizeOf(TConnector));
      SetConnector(i, TempConn.Nr);
      SetConnectorHeight(i, TempConn.Height);
    end;
    S.Read(B, SizeOf(B));
    if B then begin
      FlexAddition:=TFlexAddition.Create(CacheElement);
      FlexAddition.ModuleCache:=ModuleCache;
      FlexAddition.FromStream(S);
      CheckIn;
    end;
  except
    Result:=FALSE;
  end;
end;

//TempEbene, MyColor, ShowName, FullName are not yet handled
function TInstance.ImportWR1(var S:TStream): boolean;
var TempEbene,
    OldColor  : Byte;
    TempReal  : real;
    ShowName,
    FullName  : ByteBool;
    Number    : integer;
    TempConn  : PConnector;
    i         : longint;
    TL        : TList;

  procedure Eat(w:WORD);
  var Dummy:WORD;
  begin
    S.Read(Dummy, SizeOf(WORD));
    {$IFDEF DEBUG} WriteLn('ate ', Dummy, ' vgl ', w);{$ENDIF}
  end;

  function GetOldInt:Int16; begin S.Read(Result, 2) end;
  function GetWord:WORD; begin S.Read(Result, 2) end;

begin
  Status:=0;
  try
    ID:=GetWord;
    S.Read(TempEbene, 1);
    S.Read(TempReal, SizeOf(TempReal));  Position.X:=TempReal;
    S.Read(TempReal, SizeOf(TempReal));  Position.Y:=TempReal;
    S.Read(TempReal, SizeOf(TempReal));  Angle:=TempReal;

    S.Read(OldColor, SizeOf(OldColor));
    // colors according TO the raily for Windows 1.0x palette
    case OldColor of
      1 : Color:=clBlack;
      2 : Color:=clGray;
      3 : Color:=clGreen;
      4 : Color:=clNavy; // Should be Cyan ;
      5 : Color:=clBlue;
      6 : Color:=clFuchsia; // Should be magenta
    else Color:=clBlack; end;
    Layer:=OldColor;
    S.Read(TempReal, SizeOf(TempReal));  TextPos.X:=TempReal;
    S.Read(TempReal, SizeOf(TempReal));  TextPos.Y:=TempReal;
    S.Read(ShowName, SizeOf(ShowName));
    S.Read(FullName, SizeOf(FullName));
    if ShowName then Status:=Status or CShowRailNames;
    if FullName then Status:=Status or CShowFullNames;
    Eat(ImportCollID);  Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);
    TL:=TList.Create;
    for i:=0 TO Number-1 do begin
      Eat(ImportConnectorID);
      New(TempConn);
      TempConn.Nr:=GetWord;
      TL.Add(TempConn);  // save Connectors in A buffer (because we cannot Create
                         // the instance now)
      if S.Position>=S.Size then raise Exception.Create('');
    end;
    S.Read(ModuleName, SizeOf(ModuleName));
    S.Read(ElementName, SizeOf(ElementName));
    Result:=Recache;
    if not Result then raise Exception.Create('Element/module not found');

    for i:=0 TO Number-1 do begin
      if PConnector(TL[i])^.Nr=0 then SetConnector(i, -1)
                                 else SetConnector(i, PConnector(TL[i])^.Nr);
      SetConnectorHeight(i, 0);
      Dispose(PConnector(TL[i]));
    end;
    TL.Free;

    CheckIn;
  except
    Result:=FALSE;
  end;
end;

function TInstance.Recache:boolean;
begin
  Result:=AssignToElement(ModuleName, ElementName);
  if IsFlex then FlexAddition.ModuleCache:=ModuleCache;
end;

procedure TInstance.CheckIn;
var tUL, tLR:TDoublePoint;
begin
  tUL.X:=0; tUL.Y:=0;
  tLR.X:=0; tLR.Y:=0;
  if IsFlex then FlexAddition.CheckIn(tUL, tLR)
            else CacheElement.GetRectAround1(Position.X, Position.Y, Angle, tUL, tLR);
CheckUL.X:=tUL.X;CheckUL.Y:=tUL.Y;
CheckLR.X:=tLR.X;CheckLR.Y:=tLR.Y;
  UL.X:=tUL.X-ModuleCache.Info.ElWidth; UL.Y:=tUL.Y-ModuleCache.Info.ElWidth;
  LR.X:=tLR.X+ModuleCache.Info.ElWidth; LR.Y:=tLR.Y+ModuleCache.Info.ElWidth;
end;

function TInstance.GetLength:Double;
begin
  if IsFlex then Result:=FlexAddition.GetLength
            else Result:=CacheElement.GetLength;
end;

procedure TInstance.Draw(X: TCanvas; aX, aY, fX, fY: Double; Selected:boolean; DrawInfo:PDrawInfo);
var Prev      : integer;
    TempColor : TColor;
begin
  if not Assigned(CacheElement) then Recache;
  TempColor:=X.Pen.Color;
  DrawInfo^.Spurweite:=ModuleCache.Info.Spurweite;
  DrawInfo^.ElWidth:=ModuleCache.Info.ElWidth;

// Color Overruling
  if DrawInfo^.OverruleColor then begin
    if DrawInfo^.Blacked then X.Pen.Color:=clBlack else begin
      if GlobalPlanCheapGray then X.Pen.Color:=clGray
                             else X.Pen.Color:=TransColor(Color, clWindow, 0.80);
    end;
  end else   if Selected then begin
    X.Pen.Color:=clRed;
  end else X.Pen.Color:=Color;


{  X.MoveTo(Round((aX+UL.X)*fX),Round((aY+UL.Y)*fX));
  X.LineTo(Round((aX+LR.X)*fX),Round((aY+LR.Y)*fX));}
  if IsFlex then FlexAddition.Draw(X, aX, aY, fX, fY, DrawInfo)
            else CacheElement.Draw(X, Position.X+aX, Position.Y+aY, Angle, fX, fY, DrawInfo);

  if DrawInfo^.ShowRailNames and ((Status and CShowRailNames) >0) then begin
    if Round(ABS(GlobalRailNrHeight*fY))>5 then begin
      Prev:=SetBKMode(X.Handle, TRANSPARENT);
      if DrawInfo^.TextColorOverrule then begin
        if X.Font.Color<>clGray then X.Font.Color:=clGray;
      end else begin
        if Selected then begin
          if X.Font.Color<>clRed then X.Font.Color:=clRed;
        end else
          if X.Font.Color<>clBlack then X.Font.Color:=clBlack;
      end;
      if ((Status and CShowFullNames) >0) or (POS(' ', CacheElement.Info.Name)=0)  then
        X.TextOut(Round((aX+TextPos.X)*fX), Round((aY+TextPos.Y)*fY), CacheElement.Info.Name)
      else
        X.TextOut(Round((aX+TextPos.X)*fX), Round((aY+TextPos.Y)*fY), Copy(CacheElement.Info.Name, 1, POS(' ', CacheElement.Info.Name)));

      SetBKMode(X.Handle, Prev);
    end;
  end;
  X.Pen.Color:=TempColor;
end;

procedure TInstance.ExportLines(X: TLineExport; aX, aY, fX, fY: Double; Selected:boolean; DrawInfo:PDrawInfo);
begin
  // added by Barnoid
  X.SetPen((Layer+1) MOD X.GetPenCount);
  if (Color <> clBlack) then X.SetPen(Color MOD X.GetPenCount);
  // that'S it
  DrawInfo^.Spurweite:=ModuleCache.Info.Spurweite;
  DrawInfo^.ElWidth:=ModuleCache.Info.ElWidth;
  if IsFlex then FlexAddition.ExportLines(X, aX, aY, fX, fY, DrawInfo)
            else CacheElement.ExportIt(X, Position.X+aX, Position.Y+aY, Angle, fX, fY, DrawInfo);
  // added by Barnoid
  // wp: der ElementName enthlt ein Leerzeichen nach der Nummer !
  if GlobalPlanShowNames then begin
     if ((Status and CShowFullNames) >0) or (POS(' ', CacheElement.Info.Name)=0)  then
       X.TextOut(TextPos.X, TextPos.Y, 0, 0, ElementName, GlobalRailNrHeight div 2)
     else
       X.TextOut(TextPos.X, TextPos.Y, 0, 0, Copy(ElementName, 1, POS(' ', ElementName) - 1), GlobalRailNrHeight div 2);
  end;

  // that'S it

end;

function TInstance.IsHit(ClX, ClY: longint; aX, aY, fX, fY: Double; DrawInfo:PDrawInfo):boolean;
var Dummy:longint;
begin
  DrawInfo^.Clickbreite:=ModuleCache.Info.Spurweite/2;
  if IsFlex then Result:=FlexAddition.IsHit(ClX, ClY, aX, aY, fX, fY, DrawInfo)
            else Result:=CacheElement.ElementHit(ClX, ClY, Position.X+aX, Position.Y+aY, Angle, fX, fY, Dummy, DrawInfo)
end;

// Purpose: Sets the Position of the rail's name to a reasonable Value
// Why?   : 1. Initialisation, 2. if the users mixes 'em up
procedure TInstance.ResetNamePosition;
var P, PH:TDoublePoint;
    i:integer;
    A:Double;
begin
  P.X:=0;
  P.Y:=0;
  for i:=0 TO ConnectorCount-1 do begin
    GetConnectorPos(i, PH, A);
    P.X:=P.X+PH.X;
    P.Y:=P.Y+PH.Y;
  end;
  if ConnectorCount>0 then begin
    TextPos.X:=P.X/ConnectorCount;
    TextPos.Y:=P.Y/ConnectorCount;
  end else TextPos:=Position;
end;

procedure TInstance.RemoveConnectors;
var i:integer;
begin
  for i:=Connectors.Count-1 downto 0 do begin
    Dispose(PConnector(Connectors.Items[i]));
    Connectors.Delete(i);
  end;
end;

function TInstance.GetConnectorPos(Nr:integer; var P:TDoublePoint; var A:Double):boolean;
begin
  Result:=FALSE;
  if not Assigned(CacheElement) then Exit;
  if IsFlex then Result:=FlexAddition.GetConnectorPos(Nr, P, A)
            else Result:=CacheElement.GetConnectorWorldPos(Nr, Position.X, Position.Y, Angle, P, A);
end;

function TInstance.IsInRect(aUL, aDR: TDoublePoint):boolean;
begin
  Result:=(CheckUL.X>=aUL.X) and (CheckUL.Y>=aUL.Y) and (CheckLR.X<=aDR.X) and (CheckLR.Y<=aDR.Y);
end;


procedure TInstance.MakeFlex;
var P1, P2, PM:TDoublePoint;
    Ta1, Ta2: Double;
begin
  if IsFlex then Exit;
  // the connector positions must be found before the creation of the FlexAddition
	// because GetConnectorPos has a different meaning afterwards
  GetConnectorPos(0, P1, Ta1);
  GetConnectorPos(1, P2, Ta2);
  FlexAddition:=TFlexAddition.Create(CacheElement);
  FlexAddition.A1:=Ta1+pi;
  FlexAddition.A2:=Ta2;
  FlexAddition.ModuleCache:=ModuleCache;
  FlexAddition.Points.Add(TPointObject.Create(P1.X, P1.Y));
  if CacheElement.Info.TheType=FlexElement then begin
    PM.X:=P1.X+(P2.X-P1.X)/4;
    PM.Y:=P1.Y+(P2.Y-P1.Y)/4;
    FlexAddition.Points.Add(TPointObject.Create(PM.X, PM.Y));
    PM.X:=P1.X+(P2.X-P1.X)/2;
    PM.Y:=P1.Y+(P2.Y-P1.Y)/2;
    FlexAddition.Points.Add(TPointObject.Create(PM.X, PM.Y));
    PM.X:=P1.X+3*(P2.X-P1.X)/4;
    PM.Y:=P1.Y+3*(P2.Y-P1.Y)/4;
    FlexAddition.Points.Add(TPointObject.Create(PM.X, PM.Y));
  end;
  FlexAddition.Points.Add(TPointObject.Create(P2.X, P2.Y));
  FlexAddition.Calculate;
  CheckIn;
end;

function TInstance.IsFlex:boolean;
begin
  Result:=Assigned(FlexAddition);
end;

destructor TInstance.Free;
begin
  RemoveConnectors;
  Connectors.Free;
end;

constructor TFlexAddition.Create(X:TElement);
begin
  ElementCache:=X;
  Points:=TList.Create;
  PolyLine1:=TList.Create;
  PolyLine2:=TList.Create;
  PolyLine3:=TList.Create;
  PolyLine4:=TList.Create;
  PolyLine5:=TList.Create;
  Illegals:=TBits.Create;
end;

procedure TFlexAddition.FreePointList(var X:TList);
var i:integer;
begin
  if not Assigned(X) then Exit;
  for i:=X.Count-1 downto 0 do begin
    TPointObject(X.Items[i]).Free;
    X.Delete(i);
  end;
  X.Free; X:=NIL;
end;

destructor TFlexAddition.Free;
begin
  FreePointList(Points);
  FreePointList(PolyLine1);
  FreePointList(PolyLine2);
  FreePointList(PolyLine3);
  FreePointList(PolyLine4);
  FreePointList(PolyLine5);
  Illegals.Free;
end;

function TFlexAddition.ToStream(var S:TStream):boolean;
var i, Number:integer;
begin
  Result:=TRUE;
  Number:=Points.Count;
  S.Write(Number, SizeOf(Number));
  for i:=0 TO Number-1 do begin
    S.Write(TPointObject(Points.Items[i]).P, SizeOf(TDoublePoint));
  end;
  S.Write(A1, SizeOf(A1));
  S.Write(A2, SizeOf(A2));
end;

function TFlexAddition.FromStream(var S:TStream):boolean;
var i, Number:integer;
    P  : TDoublePoint;
begin
  Result:=TRUE;
  try
    S.Read(Number, SizeOf(Number));
    for i:=0 TO Number-1 do begin
      if S.Position>=S.Size then raise Exception.Create('');
      S.Read(P, SizeOf(TDoublePoint));
      Points.Add(TPointObject.Create(P.X, P.Y));
    end;
  except Result:=FALSE end;
  S.Read(A1, SizeOf(A1));
  S.Read(A2, SizeOf(A2));
  Calculate;
end;

procedure TFlexAddition.Turnmove(dx, dy, DeltaAngle:Double; P1:TDoublePoint);
var i:integer;

  function TMPoint(P:TDoublePoint):TDoublePoint;
  begin
     P:=GlobalRotatePoint(P, P1, -DeltaAngle);
     P.X:=P.X+dx;
     P.Y:=P.Y+dy;
     Result:=P;
  end;

begin
  A1:=Fixedangle(A1-DeltaAngle);
  A2:=Fixedangle(A2-DeltaAngle);
  for i:=0 TO Points.Count-1 do TPointObject(Points[i]).P:=TMPoint(TPointObject(Points[i]).P);
  Calculate;
end;


procedure TFlexAddition.Draw(X: TCanvas; aX, aY, fX, fY: Double; DrawInfo:PDrawInfo);
var i         : integer;
    OCol      : TColor;
    OWidth    : integer;
begin
  OCol:=X.Pen.Color;
  OWidth:=X.Pen.Width;
  if (DrawInfo.DoubleDraw and CDrawBettung) >0 then begin
    if Assigned(PolyLine4) and (PolyLine4.Count>=2) then begin
      X.MoveTo(Round(0.5+(TPointObject(PolyLine4[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine4[0]).P.Y+aY)*fY));
      for i:=1 TO PolyLine2.Count-1 do begin
        if GlobalFlexRadiusChk then begin
          if Illegals[i] then begin
            X.Pen.Color:=clRed;
            X.Pen.Width:=3;
          end else begin
            X.Pen.Color:=OCol;
            X.Pen.Width:=OWidth;
          end;
        end;
        X.LineTo(Round(0.5+(TPointObject(PolyLine4[i]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine4[i]).P.Y+aY)*fY));
      end;
    end;
    if Assigned(PolyLine5) and (PolyLine5.Count>=2) then begin
      X.MoveTo(Round(0.5+(TPointObject(PolyLine5[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine5[0]).P.Y+aY)*fY));
      for i:=1 TO PolyLine5.Count-1 do begin
        if GlobalFlexRadiusChk then begin
          if Illegals[i] then begin
            X.Pen.Color:=clRed;
            X.Pen.Width:=3;
          end else begin
            X.Pen.Color:=OCol;
            X.Pen.Width:=OWidth;
          end;
        end;
        X.LineTo(Round(0.5+(TPointObject(PolyLine5[i]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine5[i]).P.Y+aY)*fY));
      end;
    end;
    if (DrawInfo.CnctDraw and CDrawCnctSeparator)>0 then begin
      X.MoveTo(Round(0.5+(TPointObject(PolyLine5[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine5[0]).P.Y+aY)*fY));
      X.LineTo(Round(0.5+(TPointObject(PolyLine4[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine4[0]).P.Y+aY)*fY));
      X.MoveTo(Round(0.5+(TPointObject(PolyLine5[PolyLine5.Count-1]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine5[PolyLine5.Count-1]).P.Y+aY)*fY));
      X.LineTo(Round(0.5+(TPointObject(PolyLine4[PolyLine4.Count-1]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine4[PolyLine4.Count-1]).P.Y+aY)*fY));
    end;
  end;

  if (DrawInfo.DoubleDraw and CDrawOuter) >0 then begin
    if Assigned(PolyLine2) and (PolyLine2.Count>=2) then begin
      X.MoveTo(Round(0.5+(TPointObject(PolyLine2[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine2[0]).P.Y+aY)*fY));
      for i:=1 TO PolyLine2.Count-1 do begin
        if GlobalFlexRadiusChk then begin
          if Illegals[i] then begin
            X.Pen.Color:=clRed;
            X.Pen.Width:=3;
          end else begin
            X.Pen.Color:=OCol;
            X.Pen.Width:=OWidth;
          end;
        end;
        X.LineTo(Round(0.5+(TPointObject(PolyLine2[i]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine2[i]).P.Y+aY)*fY));
      end;
    end;
    if Assigned(PolyLine3) and (PolyLine3.Count>=2) then begin
      X.MoveTo(Round(0.5+(TPointObject(PolyLine3[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine3[0]).P.Y+aY)*fY));
      for i:=1 TO PolyLine3.Count-1 do begin
        if GlobalFlexRadiusChk then begin
          if Illegals[i] then begin
            X.Pen.Color:=clRed;
            X.Pen.Width:=3;
          end else begin
            X.Pen.Color:=OCol;
            X.Pen.Width:=OWidth;
          end;
        end;
        X.LineTo(Round(0.5+(TPointObject(PolyLine3[i]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine3[i]).P.Y+aY)*fY));
      end;
    end;
    if ((DrawInfo.DoubleDraw and CDrawBettung)=0) and ((DrawInfo.CnctDraw and CDrawCnctSeparator)>0) then begin
      X.MoveTo(Round(0.5+(TPointObject(PolyLine2[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine2[0]).P.Y+aY)*fY));
      X.LineTo(Round(0.5+(TPointObject(PolyLine3[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine3[0]).P.Y+aY)*fY));
      X.MoveTo(Round(0.5+(TPointObject(PolyLine2[PolyLine2.Count-1]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine2[PolyLine2.Count-1]).P.Y+aY)*fY));
      X.LineTo(Round(0.5+(TPointObject(PolyLine3[PolyLine3.Count-1]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine3[PolyLine3.Count-1]).P.Y+aY)*fY));
    end;
  end;
  if (DrawInfo.DoubleDraw and CDrawInner)>0 then begin
    if Assigned(PolyLine1) and (PolyLine1.Count>=2) then begin
      X.MoveTo(Round(0.5+(TPointObject(PolyLine1[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine1[0]).P.Y+aY)*fY));
      for i:=1 TO PolyLine1.Count-1 do begin
        if GlobalFlexRadiusChk then begin
          if Illegals[i] then begin
            X.Pen.Color:=clRed;
            X.Pen.Width:=3;
          end else begin
            X.Pen.Color:=OCol;
            X.Pen.Width:=OWidth;
          end;
        end;
        X.LineTo(Round(0.5+(TPointObject(PolyLine1[i]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine1[i]).P.Y+aY)*fY));
      end;
    end;
    if ((DrawInfo.CnctDraw and CDrawCnctSeparator)>0)
       and(Assigned(PolyLine5) and (PolyLine5.Count>=2))
       and(Assigned(PolyLine4) and (PolyLine4.Count>=2))
       and((DrawInfo.DoubleDraw and CDrawOuter)=0) then begin
      X.MoveTo(Round(0.5+(TPointObject(PolyLine5[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine5[0]).P.Y+aY)*fY));
      X.LineTo(Round(0.5+(TPointObject(PolyLine4[0]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine4[0]).P.Y+aY)*fY));
      X.MoveTo(Round(0.5+(TPointObject(PolyLine5[PolyLine5.Count-1]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine5[PolyLine5.Count-1]).P.Y+aY)*fY));
      X.LineTo(Round(0.5+(TPointObject(PolyLine4[PolyLine4.Count-1]).P.X+aX)*fX), Round(0.5+(TPointObject(PolyLine4[PolyLine4.Count-1]).P.Y+aY)*fY));
    end;

  end;
end;

procedure TFlexAddition.ExportLines(X: TLineExport; aX, aY, fX, fY: Double; DrawInfo:PDrawInfo);
var i         : integer;
begin
  if (DrawInfo.DoubleDraw and CDrawBettung) >0 then begin
    if Assigned(PolyLine4) and (PolyLine4.Count>=2) then begin
      X.MoveTo((TPointObject(PolyLine4[0]).P.X+aX)*fX, (TPointObject(PolyLine4[0]).P.Y+aY)*fY);
      for i:=1 TO PolyLine2.Count-1 do begin
        X.LineTo((TPointObject(PolyLine4[i]).P.X+aX)*fX, (TPointObject(PolyLine4[i]).P.Y+aY)*fY);
      end;
    end;
    if Assigned(PolyLine5) and (PolyLine5.Count>=2) then begin
      X.MoveTo((TPointObject(PolyLine5[0]).P.X+aX)*fX, (TPointObject(PolyLine5[0]).P.Y+aY)*fY);
      for i:=1 TO PolyLine3.Count-1 do begin
        X.LineTo((TPointObject(PolyLine5[i]).P.X+aX)*fX, (TPointObject(PolyLine5[i]).P.Y+aY)*fY);
      end;
    end;
  end;
  if (DrawInfo.DoubleDraw and CDrawOuter) >0 then begin
    if Assigned(PolyLine2) and (PolyLine2.Count>=2) then begin
      X.MoveTo((TPointObject(PolyLine2[0]).P.X+aX)*fX, (TPointObject(PolyLine2[0]).P.Y+aY)*fY);
      for i:=1 TO PolyLine2.Count-1 do begin
        X.LineTo((TPointObject(PolyLine2[i]).P.X+aX)*fX, (TPointObject(PolyLine2[i]).P.Y+aY)*fY);
      end;
    end;
    if Assigned(PolyLine3) and (PolyLine3.Count>=2) then begin
      X.MoveTo((TPointObject(PolyLine3[0]).P.X+aX)*fX, (TPointObject(PolyLine3[0]).P.Y+aY)*fY);
      for i:=1 TO PolyLine3.Count-1 do begin
        X.LineTo((TPointObject(PolyLine3[i]).P.X+aX)*fX, (TPointObject(PolyLine3[i]).P.Y+aY)*fY);
      end;
    end;
  end;
  if (DrawInfo.DoubleDraw and CDrawInner)>0 then begin
    if Assigned(PolyLine1) and (PolyLine1.Count>=2) then begin
      X.MoveTo((TPointObject(PolyLine1[0]).P.X+aX)*fX, (TPointObject(PolyLine1[0]).P.Y+aY)*fY);
      for i:=1 TO PolyLine1.Count-1 do begin
        X.LineTo((TPointObject(PolyLine1[i]).P.X+aX)*fX, (TPointObject(PolyLine1[i]).P.Y+aY)*fY);
      end;
    end;
  end;
end;

procedure TFlexAddition.AddPoint(Where:integer);
var P:TPointObject;
begin
  if Where<2 then Where:=2; if Where>Points.Count-2 then Where:=Points.Count-2;
  P:=TPointObject.Create(0.5*(TPointObject(Points[Where]).P.X+TPointObject(Points[Where+1]).P.X),
                         0.5*(TPointObject(Points[Where]).P.Y+TPointObject(Points[Where+1]).P.Y));
  Points.Insert(Where+1, P);
end;

procedure TFlexAddition.DeletePoint(Which:integer);
begin
  if (Which<2) or (Which>Points.Count-3) then Exit;
  TPointObject(Points[Which]).Free;
  Points.Delete(Which);
  Points.Pack;
end;


function TFlexAddition.IsHit(ClX, ClY: longint; aX, aY, fX, fY: Double; DrawInfo:PDrawInfo):boolean;
var P, PMI:array[0..3] of TPoint;
    PM    :TPoint;
    i     : integer;
    OP, TP: TPoint;
begin
  Result:=FALSE;
  if PolyLine1.Count=0 then Exit;
  PM.X:=ClX; PM.Y:=ClY;
  InitSearch(P, PMI, PM, Round(fX*DrawInfo^.Clickbreite));
  TP.X:=Round(0.5+(TPointObject(PolyLine1[0]).P.X+aX)*fX);
  TP.Y:=Round(0.5+(TPointObject(PolyLine1[0]).P.Y+aY)*fY);
  for i:=1 TO PolyLine1.Count-1 do begin
    OP:=TP;
    TP.X:=Round(0.5+(TPointObject(PolyLine1[i]).P.X+aX)*fX);
    TP.Y:=Round(0.5+(TPointObject(PolyLine1[i]).P.Y+aY)*fY);
    Result:=Result or LineSelected(OP, TP, PMI, P);
  end;
end;

procedure TFlexAddition.Calculate;

  function Faculty(n : WORD) : EXTENDED; register;
  var f : EXTENDED;
  begin
    f := 1;
    for n:=n downto 1 do  f := f * n;
    Faculty := f;
  end;

  function Binomial(n, k : longint) : EXTENDED;   register;
  begin
    Binomial := Faculty(n) / (Faculty(k)*Faculty(n-k));
  end;

  function BGP(i, n:integer; t:Double):Double; register;
  begin
    Result:=Binomial(n,i)*Power(t, i)*Power(1-t, n-i);
  end;

  procedure Bezier(t : Double; var X, Y : Double);
  var
    i    : integer;
  begin
    X:=0;  Y := 0;
    for i := 0 TO Points.Count-1 do begin
      X := X + BGP(i,Points.Count-1,t)*TPointObject(Points[i]).P.X;
      Y := Y + BGP(i,Points.Count-1,t)*TPointObject(Points[i]).P.Y;
    end;
  end;

  function Angle(A, B: TDoublePoint):Double; register;
  begin
    try
      Result:=ArcTan2((B.Y-A.Y), (B.X-A.X));
    except
      Result:=pi;
    end;
  end;

var TX, TY, Wink:Double;
    i : integer;
    SOrth,
    COrth: EXTENDED;
    SWHalbe:Double;
    Balbe:Double;
    LegalRadius:Double;
    P1, P2:TDoublePoint;
    PA, PB, PC, PD:TDoublePoint;
    W1, W2:Double;
begin
  FreePointList(PolyLine1);
  FreePointList(PolyLine2);
  FreePointList(PolyLine3);
  FreePointList(PolyLine4);
  FreePointList(PolyLine5);
  PolyLine1:=TList.Create;
  PolyLine2:=TList.Create;
  PolyLine3:=TList.Create;
  PolyLine4:=TList.Create;
  PolyLine5:=TList.Create;
  if GlobalBezierPoints<32 then GlobalBezierPoints:=32;
  if GlobalBezierPoints>512 then GlobalBezierPoints:=512;
  if ElementCache.Info.TheType=FlexElement then begin
    for i:=0 TO GlobalBezierPoints do begin
      Bezier(i/GlobalBezierPoints,TX, TY);
      PolyLine1.Add(TPointObject.Create(TX, TY));
    end;
  end else begin
    PolyLine1.Add(TPointObject.Create(TPointObject(Points[0]).P.X, TPointObject(Points[0]).P.Y));
    PolyLine1.Add(TPointObject.Create(TPointObject(Points[1]).P.X, TPointObject(Points[1]).P.Y));
  end;
// Checking for illegal radiuses
  Illegals.Size:=PolyLine1.Count;
  LegalRadius:=GlobalFlexLegalRad;
  SinCos(A1+pi/2, SOrth, COrth);
  for i:=1 TO PolyLine1.Count-2 do begin
    W1:=Angle(TPointObject(PolyLine1[i-1]).P, TPointObject(PolyLine1[i]).P)+pi/2;
    W2:=Angle(TPointObject(PolyLine1[i]).P, TPointObject(PolyLine1[i+1]).P)+pi/2;
    P1.X:=TPointObject(PolyLine1[i-1]).P.X+0.5*(TPointObject(PolyLine1[i]).P.X-TPointObject(PolyLine1[i-1]).P.X);
    P1.Y:=TPointObject(PolyLine1[i-1]).P.Y+0.5*(TPointObject(PolyLine1[i]).P.Y-TPointObject(PolyLine1[i-1]).P.Y);
    P2.X:=TPointObject(PolyLine1[i]).P.X+0.5*(TPointObject(PolyLine1[i+1]).P.X-TPointObject(PolyLine1[i]).P.X);
    P2.Y:=TPointObject(PolyLine1[i]).P.Y+0.5*(TPointObject(PolyLine1[i+1]).P.Y-TPointObject(PolyLine1[i]).P.Y);
    SinCos(W1, SOrth, COrth);
    PA.X:=P1.X-COrth*LegalRadius;
    PA.Y:=P1.Y-SOrth*LegalRadius;
    PB.X:=P1.X+COrth*LegalRadius;
    PB.Y:=P1.Y+SOrth*LegalRadius;
    SinCos(W2, SOrth, COrth);
    PC.X:=P2.X-COrth*LegalRadius;
    PC.Y:=P2.Y-SOrth*LegalRadius;
    PD.X:=P2.X+COrth*LegalRadius;
    PD.Y:=P2.Y+SOrth*LegalRadius;
    Illegals[i]:=FALSE;
    if EchtSchnitt(PA, PB, PC, PD, P1) then
      Illegals[i]:=CalcDist(P1, P2)<=LegalRadius;
  end;


  SWHalbe:=ModuleCache.Info.Spurweite/2;
  Balbe:=ModuleCache.Info.ElWidth/2;
  SinCos(A1+pi/2, SOrth, COrth);
  PolyLine2.Add(TPointObject.Create(TPointObject(PolyLine1[0]).P.X+COrth*SWHalbe,TPointObject(PolyLine1[0]).P.Y+SOrth*SWHalbe));
  PolyLine3.Add(TPointObject.Create(TPointObject(PolyLine1[0]).P.X-COrth*SWHalbe,TPointObject(PolyLine1[0]).P.Y-SOrth*SWHalbe));
  PolyLine4.Add(TPointObject.Create(TPointObject(PolyLine1[0]).P.X+COrth*Balbe,TPointObject(PolyLine1[0]).P.Y+SOrth*Balbe));
  PolyLine5.Add(TPointObject.Create(TPointObject(PolyLine1[0]).P.X-COrth*Balbe,TPointObject(PolyLine1[0]).P.Y-SOrth*Balbe));
  for i:=1 TO PolyLine1.Count-2 do begin
    Wink:=Angle(TPointObject(PolyLine1[i-1]).P, TPointObject(PolyLine1[i]).P)+pi/2;
    SinCos(Wink, SOrth, COrth);
    PolyLine2.Add(TPointObject.Create(TPointObject(PolyLine1[i]).P.X+COrth*SWHalbe,TPointObject(PolyLine1[i]).P.Y+SOrth*SWHalbe));
    PolyLine3.Add(TPointObject.Create(TPointObject(PolyLine1[i]).P.X-COrth*SWHalbe,TPointObject(PolyLine1[i]).P.Y-SOrth*SWHalbe));
    PolyLine4.Add(TPointObject.Create(TPointObject(PolyLine1[i]).P.X+COrth*Balbe,TPointObject(PolyLine1[i]).P.Y+SOrth*Balbe));
    PolyLine5.Add(TPointObject.Create(TPointObject(PolyLine1[i]).P.X-COrth*Balbe,TPointObject(PolyLine1[i]).P.Y-SOrth*Balbe));
  end;
  SinCos(A2+pi/2, SOrth, COrth);
  PolyLine2.Add(TPointObject.Create(TPointObject(PolyLine1[PolyLine1.Count-1]).P.X+COrth*SWHalbe,TPointObject(PolyLine1[PolyLine1.Count-1]).P.Y+SOrth*SWHalbe));
  PolyLine3.Add(TPointObject.Create(TPointObject(PolyLine1[PolyLine1.Count-1]).P.X-COrth*SWHalbe,TPointObject(PolyLine1[PolyLine1.Count-1]).P.Y-SOrth*SWHalbe));
  PolyLine4.Add(TPointObject.Create(TPointObject(PolyLine1[PolyLine1.Count-1]).P.X+COrth*Balbe,TPointObject(PolyLine1[PolyLine1.Count-1]).P.Y+SOrth*Balbe));
  PolyLine5.Add(TPointObject.Create(TPointObject(PolyLine1[PolyLine1.Count-1]).P.X-COrth*Balbe,TPointObject(PolyLine1[PolyLine1.Count-1]).P.Y-SOrth*Balbe));
end;

procedure TFlexAddition.CheckIn(var tUL, tLR:TDoublePoint);
var i     : integer;
    First : boolean;
    A     : TDoublePoint;
begin
  First:=TRUE;
  tUL.X:=0; tUL.Y:=0; // Prevent div by zeros if something fails
  tLR.X:=0; tLR.Y:=0;
  for i:=0 TO PolyLine1.Count-1 do begin
    A:=TPointObject(PolyLine1[i]).P;
    if First then begin tUL.X:=A.X;tUL.Y:=A.Y; tLR.X:=A.X;tLR.Y:=A.Y; First:=FALSE; end;
    if tUL.X>A.X then tUL.X:=A.X;
    if tUL.Y>A.Y then tUL.Y:=A.Y;
    if tLR.X<A.X then tLR.X:=A.X;
    if tLR.Y<A.Y then tLR.Y:=A.Y;
  end;
end;

function TFlexAddition.GetLength:Double;
var i     : integer;
    TP    : TDoublePoint;
    OP    : TDoublePoint;
begin
  Result:=0;
  if PolyLine1.Count<2 then Exit;
  TP:=TPointObject(PolyLine1[0]).P;
  for i:=1 TO PolyLine1.Count-1 do begin
    OP:=TP;
    TP:=TPointObject(PolyLine1[i]).P;
    Result:=Result+CalcDist(OP, TP);
  end;
end;

function TFlexAddition.GetConnectorPos(Nr:integer; var P:TDoublePoint; var A:Double):boolean;
begin
  Result:=FALSE;
  if PolyLine1.Count<2 then Exit;
  if Nr=0 then begin
    P:=TPointObject(PolyLine1[0]).P;
    A:=Fixedangle(A1+pi);
  end else begin
    P:=TPointObject(PolyLine1[PolyLine1.Count-1]).P;
    A:=Fixedangle(A2);
  end;
  Result:=TRUE;
end;


end.
