(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com, parts by Bernhard Egger
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
UNIT ULineExporter;

interface
	 USES CLASSES, Graphics;

	 const LExCapSetRectAround =  100;
				 LExCapGetPenCount   =  200;
				 LExCapSetPen        =  300;
				 LExCapSetPenWidth   =  400;
				 LExCapSetPenColor   =  500;
				 LExCapLine          =  600;
				 LExCapMoveTo        =  700;
				 LExCapLineTo        =  800;
				 LExCapLine3d        =  900;
				 LExCapMoveTo3d      = 1000;
				 LExCapLineTo3d      = 1100;
				 LExCapTextOut       = 1200;


	 type
				TLineExport = class(TObject)
					 function HasDeviceCaps(X:integer):boolean; virtual; abstract;

					 procedure CreateFile(N:string); virtual; abstract;
					 procedure CloseFile; virtual; abstract;

					 procedure SetRectAround(Left, Top, Right, Bottom:Double);virtual; abstract;

					 function GetPenCount:integer;virtual; abstract;
					 procedure SetPen(Nr:integer);virtual; abstract;
					 procedure SetPenWidth(Nr:integer; Width:Double);virtual; abstract;
					 procedure SetPenColor(Nr:integer; Color:TColor);virtual; abstract;

					 procedure Line(X1, Y1, X2, Y2:Double);virtual; abstract;
					 procedure MoveTo(X, Y:Double);virtual; abstract;
					 procedure LineTo(X, Y:Double);virtual; abstract;

					 procedure Line3d(X1, Y1, H1, X2, Y2, H2:Double);virtual; abstract;
					 procedure MoveTo3d(X, Y, H:Double);virtual; abstract;
					 procedure LineTo3d(X, Y, H:Double);virtual; abstract;

					 procedure TextOut(X, Y, A, H : Double; Text: string; TextSize: Double); virtual; abstract;
				end;


				THPGL=class(TLineExport)
					private
						F       : TextFile;
						Down    : boolean;
						XScal   : Double;
            YScal   : Double;
            MyX1    : real;
            MyY1    : Double;
            PlotX   : Word;
            PlotY   : Word;
            LastX   : integer;
            LastY   : integer;

            MoveToX : Double;
            MoveToY : Double;

             function UserX2PlotX(X: Double): integer;
             function UserY2PlotY(Y: Double): integer;

           public
             constructor Create;
             destructor Free;

             function HasDeviceCaps(X:integer):boolean; override;

             procedure CreateFile(N:string); override;
             procedure CloseFile; override;

             procedure SetRectAround(Left, Top, Right, Bottom:Double);override;

             function GetPenCount: integer;override;
             procedure SetPen(Nr:integer);override;
             procedure SetPenWidth(Nr:integer; Width:Double);override;
             procedure SetPenColor(Nr:integer; Color:TColor);override;

             procedure Line(X1, Y1, X2, Y2:Double);override;
             procedure MoveTo(X, Y:Double);override;
             procedure LineTo(X, Y:Double);override;

             procedure Line3d(X1, Y1, H1, X2, Y2, H2:Double);override;
             procedure MoveTo3d(X, Y, H:Double);override;
             procedure LineTo3d(X, Y, H:Double);override;
             procedure TextOut(X, Y, A, H : Double; Text: string; TextSize: Double); override;
        end;

        TDXF=class(TLineExport)
          private
            F         : TextFile;
            MoveToX   : Double;
            MoveToY   : Double;
            Pen       : array[1..8]of string;
            ActualPen : integer;
            FTop, FWidth : Double;

          public
            constructor Create;
            destructor Free;

            function HasDeviceCaps(X:integer):boolean; override;

            procedure CreateFile(N:string); override;
            procedure CloseFile; override;

            procedure SetRectAround(Left, Top, Right, Bottom:Double);override;

            function GetPenCount: integer;override;
            procedure SetPen(Nr:integer);override;
            procedure SetPenWidth(Nr:integer; Width:Double);override;
            procedure SetPenColor(Nr:integer; Color:TColor);override;

            procedure Line(X1, Y1, X2, Y2:Double);override;
            procedure MoveTo(X, Y:Double);override;
            procedure LineTo(X, Y:Double);override;

            procedure Line3d(X1, Y1, H1, X2, Y2, H2:Double);override;
            procedure MoveTo3d(X, Y, H:Double);override;
            procedure LineTo3d(X, Y, H:Double);override;
            procedure TextOut(X, Y, A, H : Double; Text: string; TextSize: Double); override;
       end;


implementation

//******************************************************************************
// HPGL - Line export routines
//******************************************************************************
constructor THPGL.Create; // nothing
begin
end;

function THPGL.UserX2PlotX(X: Double): integer;
begin
  Result := Round(XScal*(X-MyX1));
end;

function THPGL.UserY2PlotY(Y: Double): integer;
begin
  Result := PlotY-Round(YScal*(Y-MyY1));
end;

procedure THPGL.SetRectAround(Left, Top, Right, Bottom:Double);
begin
  PlotX:=10000;
  PlotY:=7500;
  XScal := PlotX/(Right-Left);
  YScal := PlotY/(Bottom-Top);
  MyX1:=Left;
  MyY1:=Top;
end;

procedure THPGL.CreateFile(N:string);
begin
  Down:=False;
  AssignFile(F, N);
  ReWrite(F);
  LastX:=-1;
  LastY:=-1;
  Write(F,'in;SP1;');
  Write(F,'IW ',0,',',0,',',PlotX,',',PlotY,';');
end;

procedure THPGL.CloseFile;
begin
  Write(F, 'SP0;');
  System.CloseFile(F);
end;

function THPGL.HasDeviceCaps(X:integer):boolean;
begin
  case X of
    LExCapSetRectAround : Result:=TRUE;
    LExCapGetPenCount   : Result:=TRUE;
    LExCapSetPen        : Result:=TRUE;
//    LExCapSetPenWidth   : Result:=False;
//    LExCapSetPenColor   : Result:=False;
    LExCapLine          : Result:=TRUE;
    LExCapMoveTo        : Result:=TRUE;
    LExCapLineTo        : Result:=TRUE;
//    LExCapLine3d        : Result:=False;
//    LExCapMoveTo3d      : Result:=False;
//    LExCapLineTo3d      : Result:=False;
//    LExCapTextOut       : Result:=TRUE;
  else Result:=False; end;
end;

procedure THPGL.MoveTo(X, Y:Double);
begin
  MoveToX:=X;
  MoveToY:=Y;
end;

procedure THPGL.LineTo(X, Y:Double);
begin
  Line(MoveToX, MoveToY, X, Y);
  MoveTo(X, Y);
end;

procedure THPGL.Line(X1, Y1, X2, Y2:Double);
var iX1, iY1, iX2, iY2:integer;
begin
  iX1 := UserX2PlotX(X1);
  iY1 := UserY2PlotY(Y1);
  iX2 := UserX2PlotX(X2);
  iY2 := UserY2PlotY(Y2);
  if (LastX=iX1) and (LastY=iY1)  then begin
     if Down then Write(F,'PA',iX2,',',iY2,';')
             else Write(F,'PD;PA',iX2,',',iY2,';')
  end else Write(F,'PU',iX1,',',iY1,';PD;PA' ,iX2,',',iY2,';');

  LastX := iX2;
  LastY := iY2;
  Down := TRUE;
end;

procedure THPGL.SetPen(Nr:integer);
begin
  Write(F,'SP',Chr(Byte(Nr MOD 256)+48+1),';');
end;

procedure THPGL.Line3d(X1, Y1, H1, X2, Y2, H2:Double);
begin
  Line(X1, Y1, X2, Y2);
end;

procedure THPGL.MoveTo3d(X, Y, H:Double);
begin
  MoveTo(X, Y);
end;

procedure THPGL.LineTo3d(X, Y, H:Double);
begin
  LineTo(X, Y);
end;

function THPGL.GetPenCount: integer;
begin
  Result:=8;
end;

procedure THPGL.SetPenWidth(Nr:integer; Width:Double);begin end;

procedure THPGL.SetPenColor(Nr:integer; Color:TColor);begin end;

procedure THPGL.TextOut(X, Y, A, H : Double; Text: string; TextSize: Double); begin end;


destructor THPGL.Free;
begin
end;


//******************************************************************************
// DXF - Line export routines
//******************************************************************************
constructor TDXF.Create;
begin
  Pen[1] := '0';    // BLACK
  Pen[2] := 'RED';
  Pen[3] := 'BLUE';
  Pen[4] := 'GREEN';
  Pen[5] := 'MAGENTA';
  Pen[6] := 'YELLOW';
  Pen[7] := 'CYAN';
  Pen[8] := 'DARKGRAY';
  ActualPen := 1;
end;

procedure TDXF.SetRectAround(Left, Top, Right, Bottom:Double);
begin
  if (Top < Bottom) then FTop := Top else FTop := Bottom;
  FWidth := Abs(Bottom - Top);
end;

procedure TDXF.CreateFile(N:string);
// mhsahm es ist...
begin
  AssignFile(F, N);
  ReWrite(F);
  WriteLn(F, ' 0');
  WriteLn(F, 'SECTION');
  WriteLn(F, ' 2');
  WriteLn(F, 'HEADER');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDSEC');
  // nun folgen die Tabelleneintrge
  WriteLn(F, ' 0');
  WriteLn(F, 'SECTION');
  WriteLn(F, ' 2');
  WriteLn(F, 'TABLES');
  // Tabelle VPORT
  WriteLn(F, ' 0');
  WriteLn(F, 'TABLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'VPORT');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDTAB');
  // Tabelle LTYPE (Linientypen). Nur ein Linientyp beschrieben (normale Linie)
  WriteLn(F, ' 0');
  WriteLn(F, 'TABLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'LTYPE');
  WriteLn(F, ' 70');
  WriteLn(F, '1');
  WriteLn(F, ' 0');
  WriteLn(F, 'LTYPE');
  WriteLn(F, ' 2');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 3');
  WriteLn(F, 'Solid Line');
  WriteLn(F, ' 72');
  WriteLn(F, '65');
  WriteLn(F, ' 73');
  WriteLn(F, '0');
  WriteLn(F, ' 40');
  WriteLn(F, '0.0');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDTAB');
  // Tabelle LAYER
  // folgende LAYER werden definiert:
  // 0         : standard (schwarz);
  // RED       : rot
  // BLUE      : blau
  // GREEN     : grN
  // MAGENTA   : MAGENTA
  // YELLOW    : gelb
  // CYAN      : CYAN
  // DARKGRAY  : dunkelgrau
  // LIGHTGRAY : hellgrau
  // alle LAYER verwenden den Linientyp CONTINUOUS
  WriteLn(F, ' 0');
  WriteLn(F, 'TABLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 70');
  WriteLn(F, '8');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, '0');
  WriteLn(F, ' 70');
  WriteLn(F, '0');
  WriteLn(F, ' 62');
  WriteLn(F, '7');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, 'RED');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 62');
  WriteLn(F, '1');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, 'BLUE');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 62');
  WriteLn(F, '5');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, 'GREEN');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 62');
  WriteLn(F, '3');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, 'MAGENTA');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 62');
  WriteLn(F, '6');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, 'YELLOW');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 62');
  WriteLn(F, '2');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, 'CYAN');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 62');
  WriteLn(F, '4');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, 'DARKGRAY');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 62');
  WriteLn(F, '250');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'LAYER');
  WriteLn(F, ' 2');
  WriteLn(F, 'LIGHTGRAY');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 62');
  WriteLn(F, '254');
  WriteLn(F, ' 6');
  WriteLn(F, 'CONTINUOUS');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDTAB');
  // Tabellen STYLE, VIEW, DIMSTYLE, UCS, APPID
  WriteLn(F, ' 0');
  WriteLn(F, 'TABLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'STYLE');
  WriteLn(F, ' 70');
  WriteLn(F, '1');
  WriteLn(F, ' 0');
  WriteLn(F, 'STYLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'standard');
  WriteLn(F, ' 70');
  WriteLn(F, '64');
  WriteLn(F, ' 40');
  WriteLn(F, '0.0');
  WriteLn(F, ' 41');
  WriteLn(F, '1.0');
  WriteLn(F, ' 50');
  WriteLn(F, '0.0');
  WriteLn(F, ' 71');
  WriteLn(F, '0');
  WriteLn(F, ' 42');
  WriteLn(F, '0.5');
  WriteLn(F, '  3');
  WriteLn(F, 'txt');
  WriteLn(F, ' 4');
  WriteLn(F, '');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDTAB');
  WriteLn(F, ' 0');
  WriteLn(F, 'TABLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'VIEW');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDTAB');
  WriteLn(F, ' 0');
  WriteLn(F, 'TABLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'DIMSTYLE');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDTAB');
  WriteLn(F, ' 0');
  WriteLn(F, 'TABLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'UCS');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDTAB');
  WriteLn(F, ' 0');
  WriteLn(F, 'TABLE');
  WriteLn(F, ' 2');
  WriteLn(F, 'APPID');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDTAB');
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDSEC');
  // eigentliche Eintrge folgen nun unter ENTITIES
  WriteLn(F, ' 0');
  WriteLn(F, 'SECTION');
  WriteLn(F, ' 2');
  WriteLn(F, 'ENTITIES');
end;

procedure TDXF.CloseFile;
begin
  WriteLn(F, ' 0');
  WriteLn(F, 'ENDSEC');
  WriteLn(F, ' 0');
  WriteLn(F, 'EOF');
  System.CloseFile(F);
end;

function TDXF.HasDeviceCaps(X:integer):boolean;
begin
  case X of
    LExCapSetRectAround : Result:=TRUE;
    LExCapGetPenCount   : Result:=TRUE;
    LExCapSetPen        : Result:=TRUE;
//    LExCapSetPenWidth   : Result:=False;
//    LExCapSetPenColor   : Result:=False;
    LExCapLine          : Result:=TRUE;
    LExCapMoveTo        : Result:=TRUE;
    LExCapLineTo        : Result:=TRUE;
    LExCapLine3d        : Result:=TRUE;
    LExCapMoveTo3d      : Result:=TRUE;
    LExCapLineTo3d      : Result:=TRUE;
//    LExCapTextOut       : Result:=TRUE;
  else Result:=False; end;
end;

procedure TDXF.MoveTo(X, Y:Double);
begin
  MoveToX:=X;
  MoveToY:=Y;
end;

procedure TDXF.LineTo(X, Y:Double);
begin
  Line(MoveToX, MoveToY, X, Y);
  MoveTo(X, Y);
end;

procedure TDXF.Line(X1, Y1, X2, Y2:Double);
begin
  WriteLn(F, ' 0');       // Code Fr Element
  WriteLn(F, 'Line');     //  Element ist Line
  WriteLn(F, ' 8');       // LAYER
  WriteLn(F, Pen[ActualPen]);
//WriteLn(F, ' 62');      // Code Fr Farbe
//WriteLn(F, '1');        //  Farbwert
  WriteLn(F, ' 10');      // Code Fr Start-X-Koordinate
  WriteLn(F, X1:0:1);     //  Start-X-Koordinate
  WriteLn(F, ' 20');      // Code Fr Start-Y-Koordinate
  WriteLn(F, FTop + (FWidth - Y1):0:1);     //  Start-Y-Koordinate
  WriteLn(F, ' 30');      // Code Fr Start-Z-Koordinate
  WriteLn(F, '0.0');      //  keine Hhe
  WriteLn(F, ' 11');      // Code Fr Start-X-Koordinate
  WriteLn(F, X2:0:1);     //  end-X-Koordinate
  WriteLn(F, ' 21');      // Code Fr Start-Y-Koordinate
  WriteLn(F, FTop + (FWidth - Y2):0:1);     //  end-Y-Koordinate
  WriteLn(F, ' 31');      // Code Fr end-Z-Koordinate
  WriteLn(F, '0.0');      //  keine Hhe
end;

procedure TDXF.SetPen(Nr:integer);
begin
  if (Nr >= 1) and (Nr <= GetPenCount) then ActualPen := Nr;
end;

procedure TDXF.Line3d(X1, Y1, H1, X2, Y2, H2:Double);
begin
end;

procedure TDXF.MoveTo3d(X, Y, H:Double);
begin
end;

procedure TDXF.LineTo3d(X, Y, H:Double);
begin
end;

function TDXF.GetPenCount: integer;
begin
  Result:=8;
end;

procedure TDXF.SetPenWidth(Nr:integer; Width:Double);
begin
  // naja, Dicke 1 muss gengen
end;

procedure TDXF.SetPenColor(Nr:integer; Color:TColor);
begin
  if (Nr >= 1) and (Nr <= GetPenCount) then
    case Color of
      clRed     : Pen[Nr] := 'RED';
      clBlue    : Pen[Nr] := 'BLUE';
      clGreen   : Pen[Nr] := 'GREEN';
      clPurple  : Pen[Nr] := 'MAGENTA';
      clYellow  : Pen[Nr] := 'YELLOW';
      clAqua    : Pen[Nr] := 'CYAN';        // naja, stimmt halt nicht ganz
      clDkGray  : Pen[Nr] := 'DARKGRAY';
      else        Pen[Nr] := '0';
    end;
end;

procedure TDXF.TextOut(X, Y, A, H : Double; Text: string; TextSize: Double);
begin
  WriteLn(F, ' 0');
  WriteLn(F, 'Text');
  WriteLn(F, ' 8');
  WriteLn(F, '0');
  WriteLn(F, ' 10');
  WriteLn(F, X:0:1);
  WriteLn(F, ' 20');
  WriteLn(F, FTop + (FWidth - Y) - (TextSize + 2):0:1);  // so ist's am besten
  WriteLn(F, ' 30');
  WriteLn(F, '0.0');
  WriteLn(F, ' 40');
  WriteLn(F, TextSize:0:1);
  WriteLn(F, ' 1');
  WriteLn(F, Text);
end;


destructor TDXF.Free;
begin
end;

end.
