(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit UModuleSelect;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons;

type
  TModuleSelectDLG = class(TForm)
    SrcList: TListBox;
    DstList: TListBox;
    SrcLabel: TLabel;
    DstLabel: TLabel;
    IncludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    ExAllBtn: TSpeedButton;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: integer);
    function GetFirstSelection(List: TCustomListBox): integer;
    procedure SetButtons;
    procedure FormShow(Sender: TObject);
    procedure DstListDragOver(Sender, Source: TObject; X, Y: integer;
      State: TDragState; var Accept: boolean);
    procedure DstListDragDrop(Sender, Source: TObject; X, Y: integer);
    procedure SrcListDragOver(Sender, Source: TObject; X, Y: integer;
      State: TDragState; var Accept: boolean);
    procedure SrcListDragDrop(Sender, Source: TObject; X, Y: integer);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  ModuleSelectDLG: TModuleSelectDLG;

implementation

{$R *.DFM}

procedure TModuleSelectDLG.IncludeBtnClick(Sender: TObject);
var	index: integer;
begin
	index := GetFirstSelection(SrcList);
	MoveSelected(SrcList, DstList.Items);
	SetItem(SrcList, index)
end;

procedure TModuleSelectDLG.ExcludeBtnClick(Sender: TObject);
var	index: integer;
begin
	if Sender = SrcList then Exit;
	index := GetFirstSelection(DstList);
	MoveSelected(DstList, SrcList.Items);
	SetItem(DstList, index)
end;

procedure TModuleSelectDLG.IncAllBtnClick(Sender: TObject);
var	i : integer;
begin
	if Sender=DstList then Exit;
	for i := 0 to SrcList.Items.Count - 1 do
		DstList.Items.AddObject(SrcList.Items[i],	SrcList.Items.Objects[i]);
	SrcList.Items.Clear;
	SetItem(SrcList, 0)
end;

procedure TModuleSelectDLG.ExcAllBtnClick(Sender: TObject);
var i : integer;
begin
	for i := 0 to DstList.Items.Count - 1 do
		SrcList.Items.AddObject(DstList.Items[i], DstList.Items.Objects[i]);
	DstList.Items.Clear;
	SetItem(DstList, 0)
end;

procedure TModuleSelectDLG.MoveSelected(List: TCustomListBox; Items: TStrings);
var i : integer;
begin
	for i := List.Items.Count - 1 downto 0 do
		if List.Selected[i] then begin
			Items.AddObject(List.Items[i], List.Items.Objects[i]);
			List.Items.Delete(i)
		end
end;

procedure TModuleSelectDLG.SetButtons;
var	srcEmpty, dstEmpty: boolean;
begin
	srcEmpty := srcList.Items.Count = 0;
	dstEmpty := dstList.Items.Count = 0;
	IncludeBtn.Enabled := not srcEmpty;
	IncAllBtn.Enabled := not srcEmpty;
	ExcludeBtn.Enabled := not dstEmpty;
	ExAllBtn.Enabled := not dstEmpty
end;

function TModuleSelectDLG.GetFirstSelection(List: TCustomListBox): integer;
begin
	for result := 0 to List.Items.Count - 1 do
		if List.Selected[result] then Exit;
	result := LB_ERR
end;

procedure TModuleSelectDLG.SetItem(List: TListBox; Index: integer);
var maxIndex: integer;
begin
	with List do
	begin
		SetFocus;
		maxIndex := List.Items.Count - 1;
		if Index = LB_ERR then Index := 0
		else if Index > MaxIndex then Index := MaxIndex;
		Selected[Index] := True;
	end;
	SetButtons
end;

procedure TModuleSelectDLG.FormShow(Sender: TObject);
var I, Nr:integer;
		TS:TStringList;
begin
	TS:=TStringList.Create;
	TS.Assign(SrcList.Items);
	for I:=0 to DstList.Items.Count-1 do begin
		if TS.Find(DstList.Items.Strings[I], Nr) then TS.Delete(Nr);
	end;
	SrcList.Items.Assign(TS);
	TS.Free;
	SetButtons
end;

procedure TModuleSelectDLG.DstListDragOver(Sender, Source: TObject; X,
	Y: integer; State: TDragState; var Accept: boolean);
begin
	Accept:=True
end;

procedure TModuleSelectDLG.DstListDragDrop(Sender, Source: TObject; X,
	Y: integer);
begin
	if Source = DstList then Exit;
	IncludeBtnClick(nil)
end;

procedure TModuleSelectDLG.SrcListDragOver(Sender, Source: TObject; X,
	Y: integer; State: TDragState; var Accept: boolean);
begin
	Accept := True
end;

procedure TModuleSelectDLG.SrcListDragDrop(Sender, Source: TObject; X,
	Y: integer);
begin
	if Source = SrcList then Exit;
	ExcludeBtnClick(nil)
end;

end.
