(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit UPlanUndo;

interface
                                              //Windows only for test purposes
USES Classes, Graphics, Instances, Globals, RailyTypes, Windows;

type TPrepDelInstance = class (TObject)
       DeathCandidate : integer;
     end;

     TPrepColorUndo = class (TObject)
       Candidate : integer;
       Color     : TColor;
     end;

     TPrepAddInstance = class (TObject)
        ID     : integer;
        Backup : TStream;
        constructor Create;
        destructor Free;
     end;

     TUndoState = class (TObject)
       List    : TList;
       constructor Create;
       procedure AddState(X:TObject);
       destructor Free;
     end;

     TPlanUndo = class (TObject)
       private
         OpenCount    : integer;
       public
         States       : TList;
         MaxStates    : integer;
         Current      : TUndoState;
         UndoCallBack : procedure (X:TObject) of object;

         constructor Create;
         procedure Open;
         procedure AddState(X:TObject);
         procedure Close;
         procedure Undo;
         destructor Free;
     end;

implementation


constructor TPrepAddInstance.Create;
begin
	Backup := TMemoryStream.Create
end;


destructor TPrepAddInstance.Free;
begin
	TMemoryStream(Backup).Free
end;

constructor TUndoState.Create;
begin
	List := TList.Create
end;

procedure TUndoState.AddState(X:TObject);
begin
	List.Add(X)
end;

destructor TUndoState.Free;
var i : integer;
begin
	for i := List.Count-1 downto 0 do begin
		if TObject(List[i]) is TPrepAddInstance then begin TPrepAddInstance(List[i]).Free;EXIT; end;
		if TObject(List[i]) is TPrepDelInstance then begin TPrepDelInstance(List[i]).Free;EXIT; end;
		if TObject(List[i]) is TPrepColorUndo then begin TPrepColorUndo(List[i]).Free;EXIT; end;
		List.Delete(i)
	end;
	List.Free
end;


constructor TPlanUndo.Create;
begin
	States := TList.Create;
	OpenCount := 0;
	MaxStates := 100;
end;

procedure TPlanUndo.Open;
begin
	Inc(OpenCount);
	if OpenCount = 1 then begin
		Current := TUndoState.Create;
		if States.Count > MaxStates then begin
			TUndoState(States[0]).Free;
			States.Delete(0);
			States.Pack
		end;
		States.Add(Current)
	end
end;

procedure TPlanUndo.AddState(X:TObject);
begin
// if we are not in a collection of events... get a new Current
	if OpenCount=0 then begin
		Open; // get a new current
		Dec(OpenCount) // but we dont want a collection
	end;
	Current.AddState(X)
end;

procedure TPlanUndo.Close;
begin
	Dec(OpenCount)
end;

procedure TPlanUndo.Undo;
begin
	if not Assigned(UndoCallBack) then EXIT;

	if States.Count = 0 then begin MessageBeep(MB_IconAsterisk); OpenCount:=0; EXIT; end; // we can't do anything

	while Current.List.Count>0 do begin
		UndoCallBack(TObject(Current.List[Current.List.Count-1]));
		// The callback function has taken over The object
		// so we do not need doing this
		Current.List.Delete(Current.List.Count-1)
	end;
	Current.Free;
	States.Delete(States.Count-1);
	if States.Count = 0 then EXIT; // we can't do anything
	Current := States[States.Count-1]
end;

destructor TPlanUndo.Free;
var i : integer;
begin
	for i := States.Count-1 downto 0 do begin
		TUndoState(States[i]).Free;
		States.Delete(i)
	end;
	States.Free
end;

end.
