(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit URailyFX;

interface

USES Classes, ExtCtrls, Graphics, Math, RailyTypes, Globals, Elemente;

type TEffect  = class (TObject)
              LifeTime : longint;
              TotalTime: longint;
              constructor Create(L:integer);
              procedure Perform(X:TCanvas; fx, fy, xs, ys:Double; Update:boolean); virtual; abstract;
     end;

		 TElementFadeOutFX = class (TEffect)
							Element : TElement;
							Pos     : TDoublePoint;
							w       : Double;
							DrawInfo: TDrawInfo;
							BackCol : TColor;
							ElCol   : TColor;
							constructor Create(aPos: TDoublePoint; aw: Double; C, ec:TColor; aE:TElement; DI:TDrawInfo);
							procedure Perform(X:TCanvas; fx, fy, xs, ys:Double; Update:boolean); override;
		 end;

		 TLinkedTogetherFX = class (TEffect)
							Pos     : TDoublePoint;
							R       : Double;
							Visible : boolean;
							Color   : TColor;
							Wid     : integer;
							constructor Create(aPos: TDoublePoint;aR: Double; Length, w:longint; Col:TColor);
							procedure Perform(X:TCanvas; fx, fy, xs, ys:Double; Update:boolean); override;
		 end;


		 TRailyFX = class (TObject)
						private
							MyCanvas   : TCanvas;
							EffectList : TList;
						public
							constructor Create;

							procedure AddFX(X:TEffect);
							procedure SetCanvas(X:TCanvas);
							function Tick(fx, fy, xs, ys:Double):boolean;
							procedure Redraw(fx, fy, xs, ys:Double);
							procedure ClearFX(fx, fy, xs, ys:Double);

							destructor Free;
		 end;

implementation


//******************************************************************************
// Generic Effect
//******************************************************************************
constructor TEffect.Create(L:integer);
begin
	LifeTime :=L; TotalTime:=L
end;

//******************************************************************************
// Element fade out Effect
//******************************************************************************
constructor TElementFadeOutFX.Create(aPos: TDoublePoint; aw: Double; C, ec:TColor;aE:TElement; DI:TDrawInfo);
begin
	inherited Create(GlobalFXFadeOutTime);
	Pos:=aPos;
	w:=aw;
	Element:=aE; DrawInfo:=DI;
	BackCol:=C; ElCol:=ec
end;

procedure TElementFadeOutFX.Perform(X:TCanvas;fx, fy, xs, ys:Double; Update:boolean);
var f: Single;
		OldColor:TColor;
begin
	try f:=(TotalTime-LifeTime)/TotalTime; except f:=1; end;
	if Assigned(X) and Assigned(Element) then begin
		OldColor:=X.Pen.Color;
		X.Pen.Color:=TransColor(ElCol, clWhite, f);
		Element.Draw(X, Pos.X+xs, Pos.y+ys, w, fx, fy, @DrawInfo);
		X.Pen.Color:=OldColor
	end
end;

//******************************************************************************
// Elements linked together f/X
//******************************************************************************
constructor TLinkedTogetherFX.Create(aPos: TDoublePoint; aR : Double; Length, w:longint; Col:TColor);
begin
	inherited Create(Length);
	Color := Col;	Pos := aPos; R := aR;	Wid := w;
	Visible := FALSE
end;

procedure TLinkedTogetherFX.Perform(X:TCanvas; fx, fy, xs, ys:Double; Update:boolean);
var OldColor  : TColor;
		OldMode   : TPenMode;
		OBS       : TBrushStyle;
		w         : integer;
begin
	if Update and not Visible then Exit;
	if (LifeTime=0) and not Visible then Exit;
	if not Update then Visible:=not Visible;
	OldColor := X.Pen.Color;
	X.Pen.Color := Color;
	w := X.Pen.Width;
	X.Pen.Width := Wid;
	OBS:=X.Brush.Style;
	X.Brush.Style := bsClear;
	OldMode := X.Pen.Mode;
	X.Pen.Mode := pmNotxor;
	X.Ellipse(Round((xs+Pos.X-R)*fx),Round((ys+Pos.y-R)*fy),
						Round((xs+Pos.X+R)*fx),Round((ys+Pos.y+R)*fy));

	X.Brush.Style := OBS;
	X.Pen.Mode := OldMode;
	X.Pen.Width := w;	X.Pen.Color := OldColor
end;

//******************************************************************************
// f/X handler
//******************************************************************************
constructor TRailyFX.Create;
begin
	EffectList := TList.Create
end;

procedure TRailyFX.SetCanvas(X:TCanvas);
begin
	MyCanvas := X
end;

procedure TRailyFX.AddFX(X:TEffect);
begin
	EffectList.Add(X)
end;

function TRailyFX.Tick(fx, fy, xs, ys:Double):boolean;
var i:integer;
		X:TEffect;
begin
	Result := FALSE;
	for i := EffectList.Count - 1  downto 0 do begin
		if Assigned(EffectList[i]) then begin
			X := TEffect(EffectList[i]);
			Dec(X.LifeTime);
			X.Perform(MyCanvas, fx, fy, xs, ys, FALSE);
			if X.LifeTime = 0 then begin
				Result := TRUE;
				X.Free;
				EffectList.Delete(i)
			end
		end
	end
end;

procedure TRailyFX.Redraw(fx, fy, xs, ys:Double);
var i : integer;
		X : TEffect;
begin
	for i := EffectList.Count-1  downto 0 do begin
		if Assigned(EffectList[i]) then begin
			X:=TEffect(EffectList[i]);
			X.Perform(MyCanvas, fx, fy, xs, ys, TRUE)
		end
	end
end;


procedure TRailyFX.ClearFX(fx, fy, xs, ys:Double);
var i : integer;
		X : TEffect;
begin
	for i := EffectList.Count-1 downto 0 do begin
		X := TEffect(EffectList[i]);
		X.LifeTime := 0;
		X.Perform(MyCanvas, fx, fy, xs, ys, FALSE);
		X.Free;
		EffectList.Delete(i)
	end
end;

destructor TRailyFX.Free;
begin
	ClearFX(1,1,1,1);
	EffectList.Free
end;

end.
