(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
	list of conditions and the following disclaimer in the documentation and/or
	other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
	used to endorse or promote products derived from this software without specific
	prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit USpriteFX;

interface
USES
	SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
	Forms, Dialogs, ExtCtrls, StdCtrls;

type TSprite = class (TObject)
										 Bild        : TBitmap;
										 Maske       : TBitmap;
										 Hintergrund : TBitmap;
										 DoubleBuf   : TBitmap;
										 OldX, OldY  : integer;
										 First       : boolean;

										 constructor Create;
										 procedure LoadFromFile(Name:string);
										 procedure CatchScreen(Where:TCanvas; X, Y:integer);
										 procedure Draw(Where:TCanvas; X, Y:integer);
										 procedure Hide(Where:TCanvas);
										 procedure MoveTo(Where:TCanvas; X, Y:integer);
										 destructor Free;
							 end;

implementation

constructor TSprite.Create;
begin
	Bild := TBitmap.Create;
	Maske := TBitmap.Create;
	Hintergrund := TBitmap.Create;
	OldX := 0;
	OldY := 0;
	First := True
end;

procedure TSprite.LoadFromFile(Name:string);
var OldColor:TColor;
begin
	Bild.LoadFromFile(Name);
	Maske.Width      := Bild.Width;
	Maske.Height     := Bild.Height;
	Maske.MonoChrome := True;
	Hintergrund.Width      := Bild.Width;
	Hintergrund.Height     := Bild.Height;
	OldColor:=SetBkColor(Bild.Canvas.Handle, clBlack);
	BitBlt(Maske.Canvas.Handle, 0, 0, Bild.Width, Bild.Height,
				 Bild.Canvas.Handle, 0, 0, SRCCOPY);
	SetBkColor(Bild.Canvas.Handle, OldColor)
end;

procedure TSprite.CatchScreen(Where:TCanvas; X, Y:integer);
begin
	Hintergrund.Canvas.CopyRect(Rect(0,0,Bild.Width, Bild.Height), Where, Rect(X, Y, X+Bild.Width, Y+Bild.Height));
	OldX:=X; OldY:=Y
end;

procedure TSprite.Draw(Where:TCanvas; X, Y:integer);
var OldMode:TCopyMode;
begin
	OldMode:=Where.CopyMode;
	with Where do begin
		CopyMode := cmSrcAnd;
		Draw(X,Y, Maske);
		CopyMode := cmSrcPaint;
		Draw(X,Y, Bild);
		CopyMode:=OldMode
	end
end;

procedure TSprite.Hide(Where:TCanvas);
var OldMode:TCopyMode;
begin
	if not First then begin
		OldMode:=Where.CopyMode;
		Where.CopyMode := cmSrcCopy;
		Where.Draw(OldX, OldY, Hintergrund);
		Where.CopyMode := OldMode
	end;
	First:=True
end;

procedure TSprite.MoveTo(Where:TCanvas; X, Y:integer);
var Uni:TRECT;
		OldMode:TCopyMode;
begin
	UnionRect(Uni, Rect(X, Y, X+Bild.Width, Y+Bild.Height), Rect(OldX, OldY, OldX+Bild.Width, OldY+Bild.Height));
	DoubleBuf:=TBitmap.Create;
	DoubleBuf.Width      := Uni.Right-Uni.Left;
	DoubleBuf.Height     := Uni.Bottom-Uni.Top;
	DoubleBuf.Canvas.CopyRect(Rect(0,0,DoubleBuf.Width, DoubleBuf.Height), Where, Uni);
	if not First then DoubleBuf.Canvas.Draw(OldX-Uni.Left, OldY-Uni.Top, Hintergrund);
	First:=False;
	Hintergrund.Canvas.CopyRect(Rect(0,0,Bild.Width, Bild.Height), DoubleBuf.Canvas,
	Rect(X-Uni.Left, Y-Uni.Top, X+Bild.Width-Uni.Left, Y+Bild.Height-Uni.Top));
	DoubleBuf.Canvas.CopyMode := cmSrcAnd;
	DoubleBuf.Canvas.Draw(X-Uni.Left, Y-Uni.Top, Maske);
	DoubleBuf.Canvas.CopyMode := cmSrcPaint;
	DoubleBuf.Canvas.Draw(X-Uni.Left, Y-Uni.Top, Bild);
	OldMode:=Where.CopyMode;
	Where.CopyMode := cmSrcCopy;
	Where.Draw(Uni.Left, Uni.Top, DoubleBuf);
	Where.CopyMode := OldMode;
	DoubleBuf.Free;
	DoubleBuf := nil;
	OldX := X;
	OldY := Y
end;

destructor TSprite.Free;
begin
	Bild.Free;
	Maske.Free;
	Hintergrund.Free;
	DoubleBuf.Free
end;

end.


